@@ -7,15 +7,20 @@ defmodule ModPresenceDemo do
     Ejabberd.Hooks.add(:set_presence_hook, host, __ENV__.module, :on_presence, 50)
     :ok
   end
-  
+
   def stop(host) do
     info('Stopping ejabberd module Presence Demo')
     Ejabberd.Hooks.delete(:set_presence_hook, host, __ENV__.module, :on_presence, 50)
     :ok
   end
-  
+
   def on_presence(user, _server, _resource, _packet) do
     info('Receive presence for #{user}')
     :none
-  end 
+  end
+
+  # gen_mod callbacks
+  def depends(_host, _opts), do: []
+  def mod_opt_type(_), do: []
+
 end
