@@ -5,7 +5,7 @@
 %%% Created : 27 Feb 2004 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -117,9 +117,9 @@ init({SockMod, Socket}, Opts) ->
     TLSOpts = [verify_none | TLSOpts3],
     {SockMod1, Socket1} = if TLSEnabled ->
 				 inet:setopts(Socket, [{recbuf, 8192}]),
-				 {ok, TLSSocket} = p1_tls:tcp_to_tls(Socket,
+				 {ok, TLSSocket} = fast_tls:tcp_to_tls(Socket,
 								  TLSOpts),
-				 {p1_tls, TLSSocket};
+				 {fast_tls, TLSSocket};
 			     true -> {SockMod, Socket}
 			  end,
     Captcha = case proplists:get_bool(captcha, Opts) of
@@ -328,8 +328,8 @@ get_transfer_protocol(SockMod, HostPort) ->
       {gen_tcp, []} -> {Host, 80, http};
       {gen_tcp, [Port]} ->
 	  {Host, jlib:binary_to_integer(Port), http};
-      {p1_tls, []} -> {Host, 443, https};
-      {p1_tls, [Port]} ->
+      {fast_tls, []} -> {Host, 443, https};
+      {fast_tls, [Port]} ->
 	  {Host, jlib:binary_to_integer(Port), https}
     end.
 
@@ -532,10 +532,10 @@ make_xhtml_output(State, Status, Headers, XHTML) ->
     Data = case lists:member(html, Headers) of
 	true ->
 	    iolist_to_binary([?HTML_DOCTYPE,
-		    xml:element_to_binary(XHTML)]);
+		    fxml:element_to_binary(XHTML)]);
 	_ ->
 	    iolist_to_binary([?XHTML_DOCTYPE,
-		    xml:element_to_binary(XHTML)])
+		    fxml:element_to_binary(XHTML)])
     end,
     Headers1 = case lists:keysearch(<<"Content-Type">>, 1,
 				    Headers)
@@ -753,6 +753,7 @@ code_to_phrase(503) -> <<"Service Unavailable">>;
 code_to_phrase(504) -> <<"Gateway Timeout">>;
 code_to_phrase(505) -> <<"HTTP Version Not Supported">>.
 
+-spec parse_auth(binary()) -> {binary(), binary()} | {oauth, binary(), []} | undefined.
 parse_auth(<<"Basic ", Auth64/binary>>) ->
     Auth = jlib:decode_base64(Auth64),
     %% Auth should be a string with the format: user@server:password
@@ -762,7 +763,8 @@ parse_auth(<<"Basic ", Auth64/binary>>) ->
             undefined;
         Pos ->
             {User, <<$:, Pass/binary>>} = erlang:split_binary(Auth, Pos-1),
-            {User, Pass}
+            PassUtf8 = unicode:characters_to_binary(binary_to_list(Pass), utf8),
+            {User, PassUtf8}
     end;
 parse_auth(<<"Bearer ", SToken/binary>>) ->
     Token = str:strip(SToken),
