@@ -5,7 +5,7 @@
 %%% Created :  6 Dec 2002 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -91,10 +91,10 @@
 	  "m:error></stream:stream>">>).
 
 -define(INVALID_XML_ERR,
-	xml:element_to_binary(?SERR_XML_NOT_WELL_FORMED)).
+	fxml:element_to_binary(?SERR_XML_NOT_WELL_FORMED)).
 
 -define(INVALID_NS_ERR,
-	xml:element_to_binary(?SERR_INVALID_NAMESPACE)).
+	fxml:element_to_binary(?SERR_INVALID_NAMESPACE)).
 
 %%%----------------------------------------------------------------------
 %%% API
@@ -166,9 +166,9 @@ init([{SockMod, Socket}, Opts]) ->
 
 wait_for_stream({xmlstreamstart, _Name, Attrs},
 		StateData) ->
-    case xml:get_attr_s(<<"xmlns">>, Attrs) of
+    case fxml:get_attr_s(<<"xmlns">>, Attrs) of
       <<"jabber:component:accept">> ->
-	  To = xml:get_attr_s(<<"to">>, Attrs),
+	  To = fxml:get_attr_s(<<"to">>, Attrs),
 	  Host = jid:nameprep(To),
 	  if Host == error ->
 		  Header = io_lib:format(?STREAM_HEADER,
@@ -180,7 +180,7 @@ wait_for_stream({xmlstreamstart, _Name, Attrs},
 		  {stop, normal, StateData};
 	     true ->
 		  Header = io_lib:format(?STREAM_HEADER,
-					 [StateData#state.streamid, xml:crypt(To)]),
+					 [StateData#state.streamid, fxml:crypt(To)]),
 		  send_text(StateData, Header),
 		  HostOpts = case dict:is_key(Host, StateData#state.host_opts) of
 				 true ->
@@ -212,7 +212,7 @@ wait_for_stream(closed, StateData) ->
 
 wait_for_handshake({xmlstreamelement, El}, StateData) ->
     #xmlel{name = Name, children = Els} = El,
-    case {Name, xml:get_cdata(Els)} of
+    case {Name, fxml:get_cdata(Els)} of
       {<<"handshake">>, Digest} ->
 	  case dict:find(StateData#state.host, StateData#state.host_opts) of
 	      {ok, Password} ->
@@ -222,10 +222,12 @@ wait_for_handshake({xmlstreamelement, El}, StateData) ->
 			  send_text(StateData, <<"<handshake/>">>),
 			  lists:foreach(
 			    fun (H) ->
-				    ejabberd_router:register_route(H),
+				    ejabberd_router:register_route(H, ?MYNAME),
 				    ?INFO_MSG("Route registered for service ~p~n",
-					      [H])
-			    end, dict:fetch_keys(StateData#state.host_opts)),
+					      [H]),
+				    ejabberd_hooks:run(component_connected,
+			     		[H])
+			    end, dict:fetch_keys(StateData#state.host_opts)),			  
 			  {next_state, stream_established, StateData};
 		      _ ->
 			  send_text(StateData, ?INVALID_HANDSHAKE_ERR),
@@ -250,7 +252,7 @@ wait_for_handshake(closed, StateData) ->
 stream_established({xmlstreamelement, El}, StateData) ->
     NewEl = jlib:remove_attr(<<"xmlns">>, El),
     #xmlel{name = Name, attrs = Attrs} = NewEl,
-    From = xml:get_attr_s(<<"from">>, Attrs),
+    From = fxml:get_attr_s(<<"from">>, Attrs),
     FromJID = case StateData#state.check_from of
 		%% If the admin does not want to check the from field
 		%% when accept packets from any address.
@@ -269,7 +271,7 @@ stream_established({xmlstreamelement, El}, StateData) ->
 		      _ -> error
 		    end
 	      end,
-    To = xml:get_attr_s(<<"to">>, Attrs),
+    To = fxml:get_attr_s(<<"to">>, Attrs),
     ToJID = case To of
 	      <<"">> -> error;
 	      _ -> jid:from_string(To)
@@ -280,7 +282,9 @@ stream_established({xmlstreamelement, El}, StateData) ->
 	 and (FromJID /= error) ->
 	   ejabberd_router:route(FromJID, ToJID, NewEl);
        true ->
-	   Err = jlib:make_error_reply(NewEl, ?ERR_BAD_REQUEST),
+	   Lang = fxml:get_tag_attr_s(<<"xml:lang">>, El),
+	   Txt = <<"Incorrect stanza name or from/to JID">>,
+	   Err = jlib:make_error_reply(NewEl, ?ERRT_BAD_REQUEST(Lang, Txt)),
 	   send_element(StateData, Err),
 	   error
     end,
@@ -356,11 +360,13 @@ handle_info({route, From, To, Packet}, StateName,
 	  Attrs2 =
 	      jlib:replace_from_to_attrs(jid:to_string(From),
 					 jid:to_string(To), Attrs),
-	  Text = xml:element_to_binary(#xmlel{name = Name,
+	  Text = fxml:element_to_binary(#xmlel{name = Name,
 					      attrs = Attrs2, children = Els}),
 	  send_text(StateData, Text);
       deny ->
-	  Err = jlib:make_error_reply(Packet, ?ERR_NOT_ALLOWED),
+	  Lang = fxml:get_tag_attr_s(<<"xml:lang">>, Packet),
+	  Txt = <<"Denied by ACL">>,
+	  Err = jlib:make_error_reply(Packet, ?ERRT_NOT_ALLOWED(Lang, Txt)),
 	  ejabberd_router:route_error(To, From, Err, Packet)
     end,
     {next_state, StateName, StateData};
@@ -378,7 +384,9 @@ terminate(Reason, StateName, StateData) ->
     case StateName of
       stream_established ->
 	  lists:foreach(fun (H) ->
-				ejabberd_router:unregister_route(H)
+				ejabberd_router:unregister_route(H),
+				ejabberd_hooks:run(component_disconnected,
+					[StateData#state.host, Reason])
 			end,
 			dict:fetch_keys(StateData#state.host_opts));
       _ -> ok
@@ -402,7 +410,7 @@ send_text(StateData, Text) ->
 				   Text).
 
 send_element(StateData, El) ->
-    send_text(StateData, xml:element_to_binary(El)).
+    send_text(StateData, fxml:element_to_binary(El)).
 
 new_id() -> randoms:get_string().
 
