@@ -5,7 +5,7 @@
 %%% Created : 21 Mar 2007 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -71,7 +71,7 @@ process_command(From, To, Packet) ->
 		    jid:tolower(jid:remove_resource(From)),
 		case lists:member(LFrom, get_admin_jids()) of
 		  true ->
-		      Body = xml:get_path_s(Packet,
+		      Body = fxml:get_path_s(Packet,
 					    [{elem, <<"body">>}, cdata]),
 		      spawn(fun () ->
 				    process_flag(priority, high),
@@ -186,18 +186,24 @@ process_large_heap(Pid, Info) ->
                            "much memory:~n~p~n~s",
                            [node(), Pid, Info, DetailedInfo])),
     From = jid:make(<<"">>, Host, <<"watchdog">>),
+    Hint = [#xmlel{name = <<"no-permanent-store">>,
+		   attrs = [{<<"xmlns">>, ?NS_HINTS}]}],
     lists:foreach(fun (JID) ->
-                          send_message(From, jid:make(JID), Body)
+                          send_message(From, jid:make(JID), Body, Hint)
                   end, JIDs).
 
 send_message(From, To, Body) ->
+    send_message(From, To, Body, []).
+
+send_message(From, To, Body, ExtraEls) ->
     ejabberd_router:route(From, To,
 			  #xmlel{name = <<"message">>,
 				 attrs = [{<<"type">>, <<"chat">>}],
 				 children =
 				     [#xmlel{name = <<"body">>, attrs = [],
 					     children =
-						 [{xmlcdata, Body}]}]}).
+						 [{xmlcdata, Body}]}
+				      | ExtraEls]}).
 
 get_admin_jids() ->
     ejabberd_config:get_option(
