@@ -5,7 +5,7 @@
 %%% Created : 23 Nov 2002 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -43,9 +43,9 @@
 	 get_iq_namespace/1, iq_query_info/1,
 	 iq_query_or_response_info/1, is_iq_request_type/1,
 	 iq_to_xml/1, parse_xdata_submit/1,
-	 is_standalone_chat_state/1,
+	 unwrap_carbon/1, is_standalone_chat_state/1,
 	 add_delay_info/3, add_delay_info/4,
-	 timestamp_to_iso/1, timestamp_to_iso/2,
+	 timestamp_to_legacy/1, timestamp_to_iso_basic/1, timestamp_to_iso/2,
 	 now_to_utc_string/1, now_to_local_string/1,
 	 datetime_string_to_timestamp/1,
 	 term_to_base64/1, base64_to_term/1,
@@ -54,7 +54,8 @@
 	 binary_to_integer/1, binary_to_integer/2,
 	 integer_to_binary/1, integer_to_binary/2,
 	 atom_to_binary/1, binary_to_atom/1, tuple_to_binary/1,
-	 l2i/1, i2l/1, i2l/2, queue_drop_while/2]).
+	 l2i/1, i2l/1, i2l/2, queue_drop_while/2,
+	 expr_to_term/1, term_to_expr/1]).
 
 %% The following functions are deprecated and will be removed soon
 %% Use corresponding functions from jid.erl instead
@@ -92,8 +93,8 @@ make_result_iq_reply(#xmlel{name = Name, attrs = Attrs,
 -spec make_result_iq_reply_attrs([attr()]) -> [attr()].
 
 make_result_iq_reply_attrs(Attrs) ->
-    To = xml:get_attr(<<"to">>, Attrs),
-    From = xml:get_attr(<<"from">>, Attrs),
+    To = fxml:get_attr(<<"to">>, Attrs),
+    From = fxml:get_attr(<<"from">>, Attrs),
     Attrs1 = lists:keydelete(<<"to">>, 1, Attrs),
     Attrs2 = lists:keydelete(<<"from">>, 1, Attrs1),
     Attrs3 = case To of
@@ -133,8 +134,8 @@ make_error_reply(#xmlel{name = Name, attrs = Attrs,
 -spec make_error_reply_attrs([attr()]) -> [attr()].
 
 make_error_reply_attrs(Attrs) ->
-    To = xml:get_attr(<<"to">>, Attrs),
-    From = xml:get_attr(<<"from">>, Attrs),
+    To = fxml:get_attr(<<"to">>, Attrs),
+    From = fxml:get_attr(<<"from">>, Attrs),
     Attrs1 = lists:keydelete(<<"to">>, 1, Attrs),
     Attrs2 = lists:keydelete(<<"from">>, 1, Attrs1),
     Attrs3 = case To of
@@ -159,7 +160,7 @@ make_error_element(Code, Desc) ->
 
 make_correct_from_to_attrs(From, To, Attrs) ->
     Attrs1 = lists:keydelete(<<"from">>, 1, Attrs),
-    Attrs2 = case xml:get_attr(<<"to">>, Attrs) of
+    Attrs2 = case fxml:get_attr(<<"to">>, Attrs) of
 	       {value, _} -> Attrs1;
 	       _ -> [{<<"to">>, To} | Attrs1]
 	     end,
@@ -299,36 +300,31 @@ jid_replace_resource(JID, Resource) ->
 -spec get_iq_namespace(xmlel()) -> binary().
 
 get_iq_namespace(#xmlel{name = <<"iq">>, children = Els}) ->
-    case xml:remove_cdata(Els) of
-        [#xmlel{attrs = Attrs}] -> xml:get_attr_s(<<"xmlns">>, Attrs);
+    case fxml:remove_cdata(Els) of
+        [#xmlel{attrs = Attrs}] -> fxml:get_attr_s(<<"xmlns">>, Attrs);
         _                       -> <<"">>
     end;
 get_iq_namespace(_) -> <<"">>.
 
 %%
--spec(iq_query_info/1 ::
-(
-  Xmlel :: xmlel())
-    -> iq_request() | 'reply' | 'invalid' | 'not_iq'
-).
+-spec iq_query_info(Xmlel :: xmlel()) ->
+			   iq_request() | 'reply' | 'invalid' | 'not_iq'.
 
 %% @spec (xmlelement()) -> iq() | reply | invalid | not_iq
 iq_query_info(El) -> iq_info_internal(El, request).
 
 %%
--spec(iq_query_or_response_info/1 ::
-(
-  Xmlel :: xmlel())
-    -> iq_request() | iq_reply() | 'reply' | 'invalid' | 'not_iq'
-).
+-spec iq_query_or_response_info(Xmlel :: xmlel()) ->
+				       iq_request() | iq_reply() |
+				       'reply' | 'invalid' | 'not_iq'.
 
 iq_query_or_response_info(El) ->
     iq_info_internal(El, any).
 
 iq_info_internal(#xmlel{name = <<"iq">>, attrs = Attrs, children = Els}, Filter) ->
-    ID = xml:get_attr_s(<<"id">>, Attrs),
-    Lang = xml:get_attr_s(<<"xml:lang">>, Attrs),
-    {Type, Class} = case xml:get_attr_s(<<"type">>, Attrs) of
+    ID = fxml:get_attr_s(<<"id">>, Attrs),
+    Lang = fxml:get_attr_s(<<"xml:lang">>, Attrs),
+    {Type, Class} = case fxml:get_attr_s(<<"type">>, Attrs) of
         <<"set">>    -> {set,     request};
         <<"get">>    -> {get,     request};
         <<"result">> -> {result,  reply};
@@ -336,15 +332,15 @@ iq_info_internal(#xmlel{name = <<"iq">>, attrs = Attrs, children = Els}, Filter)
         _            -> {invalid, invalid}
     end,
     if Type == invalid -> invalid; Class == request; Filter == any ->
-        FilteredEls = xml:remove_cdata(Els),
+        FilteredEls = fxml:remove_cdata(Els),
         {XMLNS, SubEl} = case {Class, FilteredEls} of
             {request, [#xmlel{attrs = Attrs2}]} ->
-                {xml:get_attr_s(<<"xmlns">>, Attrs2), hd(FilteredEls)};
+                {fxml:get_attr_s(<<"xmlns">>, Attrs2), hd(FilteredEls)};
             {reply, _} ->
                 NonErrorEls = [El || #xmlel{name = SubName} = El <- FilteredEls,
                     SubName /= <<"error">>],
                 {case NonErrorEls of
-                     [NonErrorEl] -> xml:get_tag_attr_s(<<"xmlns">>, NonErrorEl);
+                     [NonErrorEl] -> fxml:get_tag_attr_s(<<"xmlns">>, NonErrorEl);
                      _            -> <<"">>
                  end,
                  FilteredEls};
@@ -372,11 +368,7 @@ iq_type_to_string(get) -> <<"get">>;
 iq_type_to_string(result) -> <<"result">>;
 iq_type_to_string(error) -> <<"error">>.
 
--spec(iq_to_xml/1 ::
-(
-  IQ :: iq())
-    -> xmlel()
-).
+-spec iq_to_xml(IQ :: iq()) -> xmlel().
 
 iq_to_xml(#iq{id = ID, type = Type, sub_el = SubEl}) ->
     if ID /= <<"">> ->
@@ -390,16 +382,11 @@ iq_to_xml(#iq{id = ID, type = Type, sub_el = SubEl}) ->
 		  children = SubEl}
     end.
 
--spec(parse_xdata_submit/1 ::
-(
-  El :: xmlel())
-    -> [{Var::binary(), Values::[binary()]}]
-    %%
-     | 'invalid'
-).
+-spec parse_xdata_submit(El :: xmlel()) ->
+				[{Var::binary(), Values::[binary()]}] | 'invalid'.
 
 parse_xdata_submit(#xmlel{attrs = Attrs, children = Els}) ->
-    case xml:get_attr_s(<<"type">>, Attrs) of
+    case fxml:get_attr_s(<<"type">>, Attrs) of
         <<"submit">> ->
             lists:reverse(parse_xdata_fields(Els, []));
         <<"form">> -> %% This is a workaround to accept Psi's wrong forms
@@ -408,17 +395,14 @@ parse_xdata_submit(#xmlel{attrs = Attrs, children = Els}) ->
             invalid
     end.
 
--spec(parse_xdata_fields/2 ::
-(
-  Xmlels :: [xmlel() | cdata()],
-  Res    :: [{Var::binary(), Values :: [binary()]}])
-    -> [{Var::binary(), Values::[binary()]}]
-).
+-spec parse_xdata_fields(Xmlels :: [xmlel() | cdata()],
+			 Res :: [{Var::binary(), Values :: [binary()]}]) ->
+				[{Var::binary(), Values::[binary()]}].
 
 parse_xdata_fields([], Res) -> Res;
 parse_xdata_fields([#xmlel{name = <<"field">>, attrs = Attrs, children = SubEls}
   | Els], Res) ->
-    case xml:get_attr_s(<<"var">>, Attrs) of
+    case fxml:get_attr_s(<<"var">>, Attrs) of
         <<>> ->
             parse_xdata_fields(Els, Res);
         Var ->
@@ -428,16 +412,12 @@ parse_xdata_fields([#xmlel{name = <<"field">>, attrs = Attrs, children = SubEls}
 parse_xdata_fields([_ | Els], Res) ->
     parse_xdata_fields(Els, Res).
 
--spec(parse_xdata_values/2 ::
-(
-  Xmlels :: [xmlel() | cdata()],
-  Res    :: [binary()])
-    -> [binary()]
-).
+-spec parse_xdata_values(Xmlels :: [xmlel() | cdata()],
+			 Res :: [binary()]) -> [binary()].
 
 parse_xdata_values([], Res) -> Res;
 parse_xdata_values([#xmlel{name = <<"value">>, children = SubEls} | Els], Res) ->
-    Val = xml:get_cdata(SubEls),
+    Val = fxml:get_cdata(SubEls),
     parse_xdata_values(Els, [Val | Res]);
 parse_xdata_values([_ | Els], Res) ->
     parse_xdata_values(Els, Res).
@@ -446,7 +426,7 @@ parse_xdata_values([_ | Els], Res) ->
 
 rsm_decode(#iq{sub_el = SubEl}) -> rsm_decode(SubEl);
 rsm_decode(#xmlel{} = SubEl) ->
-    case xml:get_subtag(SubEl, <<"set">>) of
+    case fxml:get_subtag(SubEl, <<"set">>) of
       false -> none;
       #xmlel{name = <<"set">>, children = SubEls} ->
 	  lists:foldl(fun rsm_parse_element/2, #rsm_in{}, SubEls)
@@ -455,26 +435,26 @@ rsm_decode(#xmlel{} = SubEl) ->
 rsm_parse_element(#xmlel{name = <<"max">>, attrs = []} =
 		      Elem,
 		  RsmIn) ->
-    CountStr = xml:get_tag_cdata(Elem),
+    CountStr = fxml:get_tag_cdata(Elem),
     {Count, _} = str:to_integer(CountStr),
     RsmIn#rsm_in{max = Count};
 rsm_parse_element(#xmlel{name = <<"before">>,
 			 attrs = []} =
 		      Elem,
 		  RsmIn) ->
-    UID = xml:get_tag_cdata(Elem),
+    UID = fxml:get_tag_cdata(Elem),
     RsmIn#rsm_in{direction = before, id = UID};
 rsm_parse_element(#xmlel{name = <<"after">>,
 			 attrs = []} =
 		      Elem,
 		  RsmIn) ->
-    UID = xml:get_tag_cdata(Elem),
+    UID = fxml:get_tag_cdata(Elem),
     RsmIn#rsm_in{direction = aft, id = UID};
 rsm_parse_element(#xmlel{name = <<"index">>,
 			 attrs = []} =
 		      Elem,
 		  RsmIn) ->
-    IndexStr = xml:get_tag_cdata(Elem),
+    IndexStr = fxml:get_tag_cdata(Elem),
     {Index, _} = str:to_integer(IndexStr),
     RsmIn#rsm_in{index = Index};
 rsm_parse_element(_, RsmIn) -> RsmIn.
@@ -528,25 +508,64 @@ rsm_encode_count(Count, Arr) ->
 	    children = [{xmlcdata, i2l(Count)}]}
      | Arr].
 
+-spec unwrap_carbon(xmlel()) -> xmlel().
+
+unwrap_carbon(#xmlel{name = <<"message">>} = Stanza) ->
+    case unwrap_carbon(Stanza, <<"sent">>) of
+      #xmlel{} = Payload ->
+	  Payload;
+      false ->
+	  case unwrap_carbon(Stanza, <<"received">>) of
+	    #xmlel{} = Payload ->
+		Payload;
+	    false ->
+		Stanza
+	  end
+    end;
+unwrap_carbon(Stanza) -> Stanza.
+
+-spec unwrap_carbon(xmlel(), binary()) -> xmlel() | false.
+
+unwrap_carbon(Stanza, Direction) ->
+    case fxml:get_subtag(Stanza, Direction) of
+      #xmlel{name = Direction, attrs = Attrs} = El ->
+	  case fxml:get_attr_s(<<"xmlns">>, Attrs) of
+	    NS when NS == ?NS_CARBONS_2;
+		    NS == ?NS_CARBONS_1 ->
+		case fxml:get_subtag_with_xmlns(El, <<"forwarded">>,
+						?NS_FORWARD) of
+		  #xmlel{children = Els} ->
+		      case fxml:remove_cdata(Els) of
+			[#xmlel{} = Payload] ->
+			    Payload;
+			_ ->
+			    false
+		      end;
+		  false ->
+		      false
+		end;
+	    _NS ->
+		false
+	  end;
+      false ->
+	  false
+    end.
+
 -spec is_standalone_chat_state(xmlel()) -> boolean().
 
-is_standalone_chat_state(#xmlel{name = <<"message">>} = El) ->
-    ChatStates = [<<"active">>, <<"inactive">>, <<"gone">>, <<"composing">>,
-		  <<"paused">>],
-    Stripped =
-	lists:foldl(fun(ChatState, AccEl) ->
-			    xml:remove_subtags(AccEl, ChatState,
-					       {<<"xmlns">>, ?NS_CHATSTATES})
-		    end, El, ChatStates),
-    case Stripped of
-      #xmlel{children = [#xmlel{name = <<"thread">>}]} ->
-	  true;
-      #xmlel{children = []} ->
-	  true;
-      _ ->
+is_standalone_chat_state(Stanza) ->
+    case unwrap_carbon(Stanza) of
+      #xmlel{name = <<"message">>, children = Els} ->
+	  IgnoreNS = [?NS_CHATSTATES, ?NS_DELAY],
+	  Stripped = [El || #xmlel{name = Name, attrs = Attrs} = El <- Els,
+			    not lists:member(fxml:get_attr_s(<<"xmlns">>,
+							     Attrs),
+					     IgnoreNS),
+			    Name /= <<"thread">>],
+	  Stripped == [];
+      #xmlel{} ->
 	  false
-    end;
-is_standalone_chat_state(_El) -> false.
+    end.
 
 -spec add_delay_info(xmlel(), jid() | ljid() | binary(), erlang:timestamp())
 		     -> xmlel().
@@ -558,33 +577,8 @@ add_delay_info(El, From, Time) ->
 		     binary()) -> xmlel().
 
 add_delay_info(El, From, Time, Desc) ->
-    case xml:get_subtag_with_xmlns(El, <<"delay">>, ?NS_DELAY) of
-      false ->
-	  %% Add new tag
-	  DelayTag = create_delay_tag(Time, From, Desc),
-	  xml:append_subtags(El, [DelayTag]);
-      DelayTag ->
-	  %% Update existing tag
-	  NewDelayTag =
-	      case {xml:get_tag_cdata(DelayTag), Desc} of
-		{<<"">>, <<"">>} ->
-		    DelayTag;
-		{OldDesc, <<"">>} ->
-		    DelayTag#xmlel{children = [{xmlcdata, OldDesc}]};
-		{<<"">>, NewDesc} ->
-		    DelayTag#xmlel{children = [{xmlcdata, NewDesc}]};
-		{OldDesc, NewDesc} ->
-		    case binary:match(OldDesc, NewDesc) of
-		      nomatch ->
-			  FinalDesc = <<OldDesc/binary, ", ", NewDesc/binary>>,
-			  DelayTag#xmlel{children = [{xmlcdata, FinalDesc}]};
-		      _ ->
-			  DelayTag#xmlel{children = [{xmlcdata, OldDesc}]}
-		    end
-	      end,
-	  NewEl = xml:remove_subtags(El, <<"delay">>, {<<"xmlns">>, ?NS_DELAY}),
-	  xml:append_subtags(NewEl, [NewDelayTag])
-    end.
+    DelayTag = create_delay_tag(Time, From, Desc),
+    fxml:append_subtags(El, [DelayTag]).
 
 -spec create_delay_tag(erlang:timestamp(), jid() | ljid() | binary(), binary())
 		       -> xmlel() | error.
@@ -612,6 +606,9 @@ create_delay_tag(DateTime, Host, Desc) when is_binary(Host) ->
 %% Minutes = integer()
 -spec timestamp_to_iso(calendar:datetime(), tz()) -> {binary(), binary()}.
 
+%% This is the XEP-0082 date and time format
+%% http://xmpp.org/extensions/xep-0082.html
+
 timestamp_to_iso({{Year, Month, Day},
                   {Hour, Minute, Second}},
                  Timezone) ->
@@ -632,13 +629,22 @@ timestamp_to_iso({{Year, Month, Day},
 		      end,
     {iolist_to_binary(Timestamp_string), iolist_to_binary(Timezone_string)}.
 
--spec timestamp_to_iso(calendar:datetime()) -> binary().
 
-timestamp_to_iso({{Year, Month, Day},
+-spec timestamp_to_legacy(calendar:datetime()) -> binary().
+%% This is the jabber legacy format
+%% http://xmpp.org/extensions/xep-0091.html#time
+timestamp_to_legacy({{Year, Month, Day},
                   {Hour, Minute, Second}}) ->
     iolist_to_binary(io_lib:format("~4..0B~2..0B~2..0BT~2..0B:~2..0B:~2..0B",
                                    [Year, Month, Day, Hour, Minute, Second])).
 
+-spec timestamp_to_iso_basic(calendar:datetime()) -> binary().
+%% This is the ISO 8601 basic bormat
+timestamp_to_iso_basic({{Year, Month, Day},
+                  {Hour, Minute, Second}}) ->
+    iolist_to_binary(io_lib:format("~4..0B~2..0B~2..0BT~2..0B~2..0B~2..0B",
+                                   [Year, Month, Day, Hour, Minute, Second])).
+
 -spec now_to_utc_string(erlang:timestamp()) -> binary().
 
 now_to_utc_string({MegaSecs, Secs, MicroSecs}) ->
@@ -889,6 +895,14 @@ tuple_to_binary(T) ->
 atom_to_binary(A) ->
     erlang:atom_to_binary(A, utf8).
 
+expr_to_term(Expr) ->
+    Str = binary_to_list(<<Expr/binary, ".">>),
+    {ok, Tokens, _} = erl_scan:string(Str),
+    {ok, Term} = erl_parse:parse_term(Tokens),
+    Term.
+
+term_to_expr(Term) ->
+    list_to_binary(io_lib:print(Term)).
 
 l2i(I) when is_integer(I) -> I;
 l2i(L) when is_binary(L) -> binary_to_integer(L).
