@@ -31,7 +31,7 @@
 -include("logger.hrl").
 
 -export([start/2, stop/1, compile/1, get_cookie/0,
-	 remove_node/1, set_password/3,
+	 remove_node/1, set_password/3, check_password/3,
 	 check_password_hash/4, delete_old_users/1,
 	 delete_old_users_vhost/2, ban_account/3,
 	 num_active_users/2, num_resources/2, resource_num/3,
@@ -47,7 +47,7 @@
 	 srg_delete/2, srg_list/1, srg_get_info/2,
 	 srg_get_members/2, srg_user_add/4, srg_user_del/4,
 	 send_message/5, send_stanza/3, send_stanza_c2s/4, privacy_set/3,
-	 stats/1, stats/2, mod_opt_type/1]).
+	 stats/1, stats/2, mod_opt_type/1, get_commands_spec/0, depends/2]).
 
 
 -include("ejabberd.hrl").
@@ -61,17 +61,19 @@
 %%%
 
 start(_Host, _Opts) ->
-    ejabberd_commands:register_commands(commands()).
+    ejabberd_commands:register_commands(get_commands_spec()).
 
 stop(_Host) ->
-    ejabberd_commands:unregister_commands(commands()).
+    ejabberd_commands:unregister_commands(get_commands_spec()).
 
+depends(_Host, _Opts) ->
+    [].
 
 %%%
 %%% Register commands
 %%%
 
-commands() ->
+get_commands_spec() ->
     Vcard1FieldsString = "Some vcard field names in get/set_vcard are:\n"
 	" FN		- Full Name\n"
 	" NICKNAME	- Nickname\n"
@@ -101,89 +103,156 @@ commands() ->
 			desc = "Recompile and reload Erlang source code file",
 			module = ?MODULE, function = compile,
 			args = [{file, string}],
-			result = {res, rescode}},
+			args_example = ["/home/me/srcs/ejabberd/mod_example.erl"],
+			args_desc = ["Filename of erlang source file to compile"],
+			result = {res, rescode},
+			result_example = ok,
+			result_desc = "Status code: 0 on success, 1 otherwise"},
      #ejabberd_commands{name = get_cookie, tags = [erlang],
 			desc = "Get the Erlang cookie of this node",
 			module = ?MODULE, function = get_cookie,
 			args = [],
-			result = {cookie, string}},
+			result = {cookie, string},
+			result_example = "MWTAVMODFELNLSMYXPPD",
+			result_desc = "Erlang cookie used for authentication by ejabberd"},
      #ejabberd_commands{name = remove_node, tags = [erlang],
 			desc = "Remove an ejabberd node from Mnesia clustering config",
 			module = ?MODULE, function = remove_node,
 			args = [{node, string}],
-			result = {res, rescode}},
-
+			args_example = ["ejabberd@server2"],
+			args_desc = ["Name of erlang node to remove"],
+			result = {res, rescode},
+			result_example = ok,
+			result_desc = "Status code: 0 on success, 1 otherwise"},
      #ejabberd_commands{name = num_active_users, tags = [accounts, stats],
 			desc = "Get number of users active in the last days",
                         policy = admin,
 			module = ?MODULE, function = num_active_users,
 			args = [{host, binary}, {days, integer}],
-			result = {users, integer}},
+			args_example = [<<"myserver.com">>, 3],
+			args_desc = ["Name of host to check", "Number of days to calculate sum"],
+			result = {users, integer},
+			result_example = 123,
+			result_desc = "Number of users active on given server in last n days"},
      #ejabberd_commands{name = delete_old_users, tags = [accounts, purge],
 			desc = "Delete users that didn't log in last days, or that never logged",
 			module = ?MODULE, function = delete_old_users,
 			args = [{days, integer}],
-			result = {res, restuple}},
+			args_example = [30],
+			args_desc = ["Last login age in days of accounts that should be removed"],
+			result = {res, restuple},
+			result_example = {ok, <<"Deleted 2 users: [\"oldman@myserver.com\", \"test@myserver.com\"]">>},
+			result_desc = "Result tuple"},
      #ejabberd_commands{name = delete_old_users_vhost, tags = [accounts, purge],
 			desc = "Delete users that didn't log in last days in vhost, or that never logged",
 			module = ?MODULE, function = delete_old_users_vhost,
 			args = [{host, binary}, {days, integer}],
-			result = {res, restuple}},
-
+			args_example = [<<"myserver.com">>, 30],
+			args_desc = ["Server name",
+				     "Last login age in days of accounts that should be removed"],
+			result = {res, restuple},
+			result_example = {ok, <<"Deleted 2 users: [\"oldman@myserver.com\", \"test@myserver.com\"]">>},
+			result_desc = "Result tuple"},
      #ejabberd_commands{name = check_account, tags = [accounts],
 			desc = "Check if an account exists or not",
 			module = ejabberd_auth, function = is_user_exists,
 			args = [{user, binary}, {host, binary}],
-			result = {res, rescode}},
+			args_example = [<<"peter">>, <<"myserver.com">>],
+			args_desc = ["User name to check", "Server to check"],
+			result = {res, rescode},
+			result_example = ok,
+			result_desc = "Status code: 0 on success, 1 otherwise"},
      #ejabberd_commands{name = check_password, tags = [accounts],
 			desc = "Check if a password is correct",
-			module = ejabberd_auth, function = check_password,
+			module = ?MODULE, function = check_password,
 			args = [{user, binary}, {host, binary}, {password, binary}],
-			result = {res, rescode}},
+			args_example = [<<"peter">>, <<"myserver.com">>, <<"secret">>],
+			args_desc = ["User name to check", "Server to check", "Password to check"],
+			result = {res, rescode},
+			result_example = ok,
+			result_desc = "Status code: 0 on success, 1 otherwise"},
      #ejabberd_commands{name = check_password_hash, tags = [accounts],
 			desc = "Check if the password hash is correct",
 			longdesc = "Allowed hash methods: md5, sha.",
 			module = ?MODULE, function = check_password_hash,
-			args = [{user, binary}, {host, binary}, {passwordhash, string}, {hashmethod, string}],
-			result = {res, rescode}},
+			args = [{user, binary}, {host, binary}, {passwordhash, string},
+				{hashmethod, string}],
+			args_example = [<<"peter">>, <<"myserver.com">>,
+					<<"5ebe2294ecd0e0f08eab7690d2a6ee69">>, <<"md5">>],
+			args_desc = ["User name to check", "Server to check",
+				     "Password's hash value", "Name of hash method"],
+			result = {res, rescode},
+			result_example = ok,
+			result_desc = "Status code: 0 on success, 1 otherwise"},
      #ejabberd_commands{name = change_password, tags = [accounts],
 			desc = "Change the password of an account",
 			module = ?MODULE, function = set_password,
 			args = [{user, binary}, {host, binary}, {newpass, binary}],
-			result = {res, rescode}},
+			args_example = [<<"peter">>, <<"myserver.com">>, <<"blank">>],
+			args_desc = ["User name", "Server name",
+				     "New password for user"],
+			result = {res, rescode},
+			result_example = ok,
+			result_desc = "Status code: 0 on success, 1 otherwise"},
      #ejabberd_commands{name = ban_account, tags = [accounts],
 			desc = "Ban an account: kick sessions and set random password",
 			module = ?MODULE, function = ban_account,
 			args = [{user, binary}, {host, binary}, {reason, binary}],
-			result = {res, rescode}},
-
+			args_example = [<<"attacker">>, <<"myserver.com">>, <<"Spaming other users">>],
+			args_desc = ["User name to ban", "Server name",
+				     "Reason for banning user"],
+			result = {res, rescode},
+			result_example = ok,
+			result_desc = "Status code: 0 on success, 1 otherwise"},
      #ejabberd_commands{name = num_resources, tags = [session],
 			desc = "Get the number of resources of a user",
 			module = ?MODULE, function = num_resources,
 			args = [{user, binary}, {host, binary}],
-			result = {resources, integer}},
+			args_example = [<<"peter">>, <<"myserver.com">>],
+			args_desc = ["User name", "Server name"],
+			result = {resources, integer},
+			result_example = 5,
+			result_desc = "Number of active resources for a user"},
      #ejabberd_commands{name = resource_num, tags = [session],
 			desc = "Resource string of a session number",
 			module = ?MODULE, function = resource_num,
 			args = [{user, binary}, {host, binary}, {num, integer}],
-			result = {resource, string}},
+			args_example = [<<"peter">>, <<"myserver.com">>, 2],
+			args_desc = ["User name", "Server name", "ID of resource to return"],
+			result = {resource, string},
+			result_example = <<"Psi">>,
+			result_desc = "Name of user resource"},
      #ejabberd_commands{name = kick_session, tags = [session],
 			desc = "Kick a user session",
 			module = ?MODULE, function = kick_session,
 			args = [{user, binary}, {host, binary}, {resource, binary}, {reason, binary}],
-			result = {res, rescode}},
+			args_example = [<<"peter">>, <<"myserver.com">>, <<"Psi">>,
+					<<"Stuck connection">>],
+			args_desc = ["User name", "Server name", "User's resource",
+				     "Reason for closing session"],
+			result = {res, rescode},
+			result_example = ok,
+			result_desc = "Status code: 0 on success, 1 otherwise"},
      #ejabberd_commands{name = status_num_host, tags = [session, stats],
 			desc = "Number of logged users with this status in host",
                         policy = admin,
 			module = ?MODULE, function = status_num,
 			args = [{host, binary}, {status, binary}],
-			result = {users, integer}},
+			args_example = [<<"myserver.com">>, <<"dnd">>],
+			args_desc = ["Server name", "Status type to check"],
+			result = {users, integer},
+			result_example = 23,
+			result_desc = "Number of connected sessions with given status type"},
      #ejabberd_commands{name = status_num, tags = [session, stats],
 			desc = "Number of logged users with this status",
                         policy = admin,
 			module = ?MODULE, function = status_num,
 			args = [{status, binary}],
-			result = {users, integer}},
+			args_example = [<<"dnd">>],
+			args_desc = ["Status type to check"],
+			result = {users, integer},
+			result_example = 23,
+			result_desc = "Number of connected sessions with given status type"},
      #ejabberd_commands{name = status_list_host, tags = [session],
 			desc = "List of users logged in host with their statuses",
 			module = ?MODULE, function = status_list,
@@ -464,9 +533,9 @@ commands() ->
 			tags = [offline],
 			desc = "Get the number of unread offline messages",
                         policy = user,
-			module = mod_offline, function = get_queue_length,
+			module = mod_offline, function = count_offline_messages,
 			args = [],
-			result = {res, integer}},
+			result = {value, integer}},
      #ejabberd_commands{name = send_message, tags = [stanza],
 			desc = "Send a message to a local or remote bare of full JID",
 			module = ?MODULE, function = send_message,
@@ -490,7 +559,7 @@ commands() ->
 			result = {res, rescode}},
 
      #ejabberd_commands{name = stats, tags = [stats],
-			desc = "Get statistical value: registeredusers onlineusers onlineusersnode uptimeseconds",
+			desc = "Get statistical value: registeredusers onlineusers onlineusersnode uptimeseconds processes",
                         policy = admin,
 			module = ?MODULE, function = stats,
 			args = [{name, binary}],
@@ -523,36 +592,38 @@ remove_node(Node) ->
 %%%
 
 set_password(User, Host, Password) ->
-    case ejabberd_auth:set_password(User, Host, Password) of
-	ok ->
-	    ok;
-	_ ->
-	    error
-    end.
+    Fun = fun () -> ejabberd_auth:set_password(User, Host, Password) end,
+    user_action(User, Host, Fun, ok).
+
+check_password(User, Host, Password) ->
+    ejabberd_auth:check_password(User, <<>>, Host, Password).
 
 %% Copied some code from ejabberd_commands.erl
 check_password_hash(User, Host, PasswordHash, HashMethod) ->
     AccountPass = ejabberd_auth:get_password_s(User, Host),
     AccountPassHash = case {AccountPass, HashMethod} of
 			  {A, _} when is_tuple(A) -> scrammed;
-			  {_, "md5"} -> get_md5(AccountPass);
-			  {_, "sha"} -> get_sha(AccountPass);
-			  _ -> undefined
+			  {_, <<"md5">>} -> get_md5(AccountPass);
+			  {_, <<"sha">>} -> get_sha(AccountPass);
+                          {_, Method} ->
+			      ?ERROR_MSG("check_password_hash called "
+ 					 "with hash method: ~p", [Method]),
+ 			      undefined
 		      end,
     case AccountPassHash of
 	scrammed ->
-	    ?ERROR_MSG("Passwords are scrammed, and check_password_hash can not work.", []),
+	    ?ERROR_MSG("Passwords are scrammed, and check_password_hash cannot work.", []),
 	    throw(passwords_scrammed_command_cannot_work);
-	undefined -> error;
+	undefined -> throw(unkown_hash_method);
 	PasswordHash -> ok;
-	_ -> error
+	_ -> false
     end.
 get_md5(AccountPass) ->
-    lists:flatten([io_lib:format("~.16B", [X])
-		   || X <- binary_to_list(erlang:md5(AccountPass))]).
+    iolist_to_binary([io_lib:format("~2.16.0B", [X])
+                      || X <- binary_to_list(erlang:md5(AccountPass))]).
 get_sha(AccountPass) ->
-    lists:flatten([io_lib:format("~.16B", [X])
-		   || X <- binary_to_list(p1_sha:sha1(AccountPass))]).
+    iolist_to_binary([io_lib:format("~2.16.0B", [X])
+ 		      || X <- binary_to_list(p1_sha:sha1(AccountPass))]).
 
 num_active_users(Host, Days) ->
     list_last_activity(Host, true, Days).
@@ -681,28 +752,14 @@ kick_sessions(User, Server, Reason) ->
       fun(Resource) ->
 	      kick_this_session(User, Server, Resource, Reason)
       end,
-      get_resources(User, Server)).
-
-get_resources(User, Server) ->
-    lists:map(
-      fun(Session) ->
-	      element(3, Session#session.usr)
-      end,
-      get_sessions(User, Server)).
-
-get_sessions(User, Server) ->
-    LUser = jid:nodeprep(User),
-    LServer = jid:nameprep(Server),
-    Sessions =  mnesia:dirty_index_read(session, {LUser, LServer}, #session.us),
-    true = is_list(Sessions),
-    Sessions.
+      ejabberd_sm:get_user_resources(User, Server)).
 
 set_random_password(User, Server, Reason) ->
     NewPass = build_random_password(Reason),
     set_password_auth(User, Server, NewPass).
 
 build_random_password(Reason) ->
-    Date = jlib:timestamp_to_iso(calendar:universal_time()),
+    Date = jlib:timestamp_to_legacy(calendar:universal_time()),
     RandomString = randoms:get_string(),
     <<"BANNED_ACCOUNT--", Date/binary, "--", RandomString/binary, "--", Reason/binary>>.
 
@@ -729,7 +786,8 @@ resource_num(User, Host, Num) ->
 	true ->
 	    lists:nth(Num, Resources);
 	false ->
-	    lists:flatten(io_lib:format("Error: Wrong resource number: ~p", [Num]))
+            throw({bad_argument,
+                   lists:flatten(io_lib:format("Wrong resource number: ~p", [Num]))})
     end.
 
 kick_session(User, Server, Resource, ReasonText) ->
@@ -794,7 +852,8 @@ connected_users_info() ->
 			      PI when is_integer(PI) -> PI;
 			      _ -> nil
 			  end,
-	      {[U, $@, S, $/, R], atom_to_list(Conn), IPS, Port, PriorityI, NodeS, Uptime}
+	      {binary_to_list(<<U/binary, $@, S/binary, $/, R/binary>>),
+	       atom_to_list(Conn), IPS, Port, PriorityI, NodeS, Uptime}
       end,
       USRIs).
 
@@ -804,28 +863,41 @@ connected_users_vhost(Host) ->
 
 %% Code copied from ejabberd_sm.erl and customized
 dirty_get_sessions_list2() ->
-    mnesia:dirty_select(
-      session,
-      [{#session{usr = '$1', sid = '$2', priority = '$3', info = '$4', _ = '_'},
-	[],
-	[['$1', '$2', '$3', '$4']]}]).
+    Ss = mnesia:dirty_select(
+	   session,
+	   [{#session{usr = '$1', sid = '$2', priority = '$3', info = '$4',
+		      _ = '_'},
+	     [],
+	     [['$1', '$2', '$3', '$4']]}]),
+    lists:filter(fun([_USR, _SID, _Priority, Info]) ->
+			 not proplists:get_bool(offline, Info)
+		 end, Ss).
 
 %% Make string more print-friendly
 stringize(String) ->
     %% Replace newline characters with other code
     ejabberd_regexp:greplace(String, <<"\n">>, <<"\\n">>).
 
+set_presence(User, Host, Resource, Type, Show, Status, Priority)
+        when is_integer(Priority) ->
+    BPriority = integer_to_binary(Priority),
+    set_presence(User, Host, Resource, Type, Show, Status, BPriority);
 set_presence(User, Host, Resource, Type, Show, Status, Priority) ->
-    Pid = ejabberd_sm:get_session_pid(User, Host, Resource),
-    USR = jid:to_string(jid:make(User, Host, Resource)),
-    US = jid:to_string(jid:make(User, Host, <<>>)),
-    Message = {route_xmlstreamelement,
-	       {xmlel, <<"presence">>,
-		[{<<"from">>, USR}, {<<"to">>, US}, {<<"type">>, Type}],
-		[{xmlel, <<"show">>, [], [{xmlcdata, Show}]},
-		 {xmlel, <<"status">>, [], [{xmlcdata, Status}]},
-		 {xmlel, <<"priority">>, [], [{xmlcdata, Priority}]}]}},
-    Pid ! Message.
+    case ejabberd_sm:get_session_pid(User, Host, Resource) of
+	none ->
+	    error;
+	Pid ->
+	    USR = jid:to_string(jid:make(User, Host, Resource)),
+	    US = jid:to_string(jid:make(User, Host, <<>>)),
+	    Message = {route_xmlstreamelement,
+		    {xmlel, <<"presence">>,
+			[{<<"from">>, USR}, {<<"to">>, US}, {<<"type">>, Type}],
+			[{xmlel, <<"show">>, [], [{xmlcdata, Show}]},
+			{xmlel, <<"status">>, [], [{xmlcdata, Status}]},
+			{xmlel, <<"priority">>, [], [{xmlcdata, Priority}]}]}},
+	    Pid ! Message,
+	    ok
+    end.
 
 user_sessions_info(User, Host) ->
     CurrentSec = calendar:datetime_to_gregorian_seconds({date(), time()}),
@@ -834,7 +906,9 @@ user_sessions_info(User, Host) ->
 		   {'EXIT', _Reason} ->
 		       [];
 		   Ss ->
-		       Ss
+		       lists:filter(fun(#session{info = Info}) ->
+					    not proplists:get_bool(offline, Info)
+				    end, Ss)
 	       end,
     lists:map(
       fun(Session) ->
@@ -912,7 +986,7 @@ get_vcard_content(User, Server, Data) ->
 	[_|_] ->
 	    case get_vcard(Data, A1) of
 		[false] -> throw(error_no_value_found_in_vcard);
-		ElemList -> ?DEBUG("ELS ~p", [ElemList]), [xml:get_tag_cdata(Elem) || Elem <- ElemList]
+		ElemList -> ?DEBUG("ELS ~p", [ElemList]), [fxml:get_tag_cdata(Elem) || Elem <- ElemList]
 	    end;
 	[] ->
 	    throw(error_no_vcard_found)
@@ -933,7 +1007,7 @@ get_vcard([Data], A1) ->
     get_subtag(A1, Data).
 
 get_subtag(Xmlelement, Name) ->
-    [xml:get_subtag(Xmlelement, Name)].
+    [fxml:get_subtag(Xmlelement, Name)].
 
 set_vcard_content(User, Server, Data, SomeContent) ->
     ContentList = case SomeContent of
@@ -963,7 +1037,7 @@ set_vcard_content(User, Server, Data, SomeContent) ->
 
 take_vcard_tel(TelType, [{xmlel, <<"TEL">>, _, SubEls}=OldEl | OldEls], NewEls, Taken) ->
     {Taken2, NewEls2} = case lists:keymember(TelType, 2, SubEls) of
-	true -> {xml:get_subtag(OldEl, <<"NUMBER">>), NewEls};
+	true -> {fxml:get_subtag(OldEl, <<"NUMBER">>), NewEls};
 	false -> {Taken, [OldEl | NewEls]}
     end,
     take_vcard_tel(TelType, OldEls, NewEls2, Taken2);
@@ -1097,7 +1171,8 @@ subscribe_roster({Name, Server, Group, Nick}, [{Name, Server, _, _} | Roster]) -
     subscribe_roster({Name, Server, Group, Nick}, Roster);
 %% Subscribe Name2 to Name1
 subscribe_roster({Name1, Server1, Group1, Nick1}, [{Name2, Server2, Group2, Nick2} | Roster]) ->
-    subscribe(Name1, Server1, Name2, Server2, Nick2, Group2, <<"both">>, []),
+    subscribe(Name1, Server1, iolist_to_binary(Name2), iolist_to_binary(Server2),
+	iolist_to_binary(Nick2), iolist_to_binary(Group2), <<"both">>, []),
     subscribe_roster({Name1, Server1, Group1, Nick1}, Roster).
 
 push_alltoall(S, G) ->
@@ -1126,7 +1201,7 @@ push_roster_item(LU, LS, R, U, S, Action) ->
     ejabberd_sm:route(LJID, LJID, BroadcastEl),
     Item = build_roster_item(U, S, Action),
     ResIQ = build_iq_roster_push(Item),
-    ejabberd_router:route(LJID, LJID, ResIQ).
+    ejabberd_router:route(jid:remove_resource(LJID), LJID, ResIQ).
 
 build_roster_item(U, S, {add, Nick, Subs, Group}) ->
     {xmlel, <<"item">>,
@@ -1206,10 +1281,10 @@ private_get(Username, Host, Element, Ns) ->
     [{xmlel, <<"query">>,
       [{<<"xmlns">>, ?NS_PRIVATE}],
       [SubEl]}] = ResIq#iq.sub_el,
-    binary_to_list(xml:element_to_binary(SubEl)).
+    binary_to_list(fxml:element_to_binary(SubEl)).
 
 private_set(Username, Host, ElementString) ->
-    case xml_stream:parse_element(ElementString) of
+    case fxml_stream:parse_element(ElementString) of
 	{error, Error} ->
 	    io:format("Error found parsing the element:~n  ~p~nError: ~p~n",
 		      [ElementString, Error]),
@@ -1255,8 +1330,7 @@ srg_get_info(Group, Host) ->
 	Os when is_list(Os) -> Os;
 	error -> []
     end,
-    [{jlib:atom_to_binary(Title),
-      io_lib:format("~p", [btl(Value)])} || {Title, Value} <- Opts].
+    [{jlib:atom_to_binary(Title), btl(Value)} || {Title, Value} <- Opts].
 
 btl([]) -> [];
 btl([B|L]) -> [btl(B)|btl(L)];
@@ -1332,21 +1406,33 @@ build_packet(Type, Subject, Body) ->
      [{xmlel, <<"body">>, [], [{xmlcdata, Body}]} | Tail]
     }.
 
-send_stanza(FromString, ToString, Data) ->
-    Stanza = {xmlel, _, _, _} = xml_stream:parse_element(Data),
-    From = jid:from_string(FromString),
-    To = jid:from_string(ToString),
-    ejabberd_router:route(From, To, Stanza).
+send_stanza(FromString, ToString, Stanza) ->
+    case fxml_stream:parse_element(Stanza) of
+	{error, Error} ->
+	    {error, Error};
+	XmlEl ->
+	    #xmlel{attrs = Attrs} = XmlEl,
+	    From = jid:from_string(proplists:get_value(<<"from">>, Attrs, FromString)),
+	    To = jid:from_string(proplists:get_value(<<"to">>, Attrs, ToString)),
+	    ejabberd_router:route(From, To, XmlEl)
+    end.
 
 send_stanza_c2s(Username, Host, Resource, Stanza) ->
-    C2sPid = ejabberd_sm:get_session_pid(Username, Host, Resource),
-    XmlEl = xml_stream:parse_element(Stanza),
-    p1_fsm:send_event(C2sPid, {xmlstreamelement, XmlEl}).
+    case {fxml_stream:parse_element(Stanza),
+          ejabberd_sm:get_session_pid(Username, Host, Resource)}
+    of
+	{{error, Error}, _} ->
+	    {error, Error};
+	{_, none} ->
+	    {error, no_session};
+	{XmlEl, C2sPid} ->
+	    p1_fsm:send_event(C2sPid, {xmlstreamelement, XmlEl})
+    end.
 
 privacy_set(Username, Host, QueryS) ->
     From = jid:make(Username, Host, <<"">>),
     To = jid:make(<<"">>, Host, <<"">>),
-    QueryEl = xml_stream:parse_element(QueryS),
+    QueryEl = fxml_stream:parse_element(QueryS),
     StanzaEl = {xmlel, <<"iq">>, [{<<"type">>, <<"set">>}], [QueryEl]},
     IQ = jlib:iq_query_info(StanzaEl),
     ejabberd_hooks:run_fold(
@@ -1364,6 +1450,7 @@ privacy_set(Username, Host, QueryS) ->
 stats(Name) ->
     case Name of
 	<<"uptimeseconds">> -> trunc(element(1, erlang:statistics(wall_clock))/1000);
+	<<"processes">> -> length(erlang:processes());
 	<<"registeredusers">> -> lists:foldl(fun(Host, Sum) -> ejabberd_auth:get_vh_registered_users_number(Host) + Sum end, 0, ?MYHOSTS);
 	<<"onlineusersnode">> -> length(ejabberd_sm:dirty_get_my_sessions_list());
 	<<"onlineusers">> -> length(ejabberd_sm:dirty_get_sessions_list())
@@ -1502,6 +1589,20 @@ decide_rip_jid({UName, UServer}, Match_list) ->
       end,
       Match_list).
 
+user_action(User, Server, Fun, OK) ->
+    case ejabberd_auth:is_user_exists(User, Server) of
+        true ->
+ 	    case catch Fun() of
+                OK -> ok;
+ 		{error, Error} -> throw(Error);
+                Error ->
+                    ?ERROR_MSG("Command returned: ~p", [Error]),
+ 		    1
+ 	    end;
+ 	false ->
+ 	    throw({not_found, "unknown_user"})
+    end.
+
 %% Copied from ejabberd-2.0.0/src/acl.erl
 is_regexp_match(String, RegExp) ->
     case ejabberd_regexp:run(String, RegExp) of
