@@ -5,7 +5,7 @@
 %%% Created : 7 Oct 2006 by Magnus Henoch <henoch@dtek.chalmers.se>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -41,7 +41,7 @@
          import_start/2, import_stop/2]).
 
 %% gen_mod callbacks
--export([start/2, start_link/2, stop/1]).
+-export([start/2, start_link/2, stop/1, depends/2]).
 
 %% gen_server callbacks
 -export([init/1, handle_info/2, handle_call/3,
@@ -80,6 +80,12 @@
 
 -record(state, {host = <<"">> :: binary()}).
 
+-callback init(binary(), gen_mod:opts()) -> any().
+-callback caps_read(binary(), {binary(), binary()}) ->
+    {ok, non_neg_integer() | [binary()]} | error.
+-callback caps_write(binary(), {binary(), binary()},
+		     non_neg_integer() | [binary()]) -> any().
+
 start_link(Host, Opts) ->
     Proc = gen_mod:get_module_proc(Host, ?PROCNAME),
     gen_server:start_link({local, Proc}, ?MODULE,
@@ -120,12 +126,12 @@ read_caps(Els) -> read_caps(Els, nothing).
 read_caps([#xmlel{name = <<"c">>, attrs = Attrs}
 	   | Tail],
 	  Result) ->
-    case xml:get_attr_s(<<"xmlns">>, Attrs) of
+    case fxml:get_attr_s(<<"xmlns">>, Attrs) of
       ?NS_CAPS ->
-	  Node = xml:get_attr_s(<<"node">>, Attrs),
-	  Version = xml:get_attr_s(<<"ver">>, Attrs),
-	  Hash = xml:get_attr_s(<<"hash">>, Attrs),
-	  Exts = str:tokens(xml:get_attr_s(<<"ext">>, Attrs),
+	  Node = fxml:get_attr_s(<<"node">>, Attrs),
+	  Version = fxml:get_attr_s(<<"ver">>, Attrs),
+	  Hash = fxml:get_attr_s(<<"hash">>, Attrs),
+	  Exts = str:tokens(fxml:get_attr_s(<<"ext">>, Attrs),
 			    <<" ">>),
 	  read_caps(Tail,
 		    #caps{node = Node, hash = Hash, version = Version,
@@ -135,7 +141,7 @@ read_caps([#xmlel{name = <<"c">>, attrs = Attrs}
 read_caps([#xmlel{name = <<"x">>, attrs = Attrs}
 	   | Tail],
 	  Result) ->
-    case xml:get_attr_s(<<"xmlns">>, Attrs) of
+    case fxml:get_attr_s(<<"xmlns">>, Attrs) of
       ?NS_MUC_USER -> nothing;
       _ -> read_caps(Tail, Result)
     end;
@@ -149,7 +155,7 @@ user_send_packet(#xmlel{name = <<"presence">>, attrs = Attrs,
 		 #jid{luser = User, lserver = Server} = From,
 		 #jid{luser = User, lserver = Server,
 		      lresource = <<"">>}) ->
-    Type = xml:get_attr_s(<<"type">>, Attrs),
+    Type = fxml:get_attr_s(<<"type">>, Attrs),
     if Type == <<"">>; Type == <<"available">> ->
 	   case read_caps(Els) of
 	     nothing -> ok;
@@ -167,7 +173,7 @@ user_receive_packet(#xmlel{name = <<"presence">>, attrs = Attrs,
 		    _C2SState,
 		    #jid{lserver = Server},
 		    From, _To) ->
-    Type = xml:get_attr_s(<<"type">>, Attrs),
+    Type = fxml:get_attr_s(<<"type">>, Attrs),
     IsRemote = not lists:member(From#jid.lserver, ?MYHOSTS),
     if IsRemote and
 	 ((Type == <<"">>) or (Type == <<"available">>)) ->
@@ -227,7 +233,7 @@ disco_info(Acc, Host, Module, Node, Lang) ->
 
 c2s_presence_in(C2SState,
 		{From, To, {_, _, Attrs, Els}}) ->
-    Type = xml:get_attr_s(<<"type">>, Attrs),
+    Type = fxml:get_attr_s(<<"type">>, Attrs),
     Subscription = ejabberd_c2s:get_subscription(From,
 						 C2SState),
     Insert = ((Type == <<"">>) or (Type == <<"available">>))
@@ -300,28 +306,12 @@ c2s_broadcast_recipients(InAcc, Host, C2SState,
     end;
 c2s_broadcast_recipients(Acc, _, _, _, _, _) -> Acc.
 
-init_db(mnesia, _Host) ->
-    case catch mnesia:table_info(caps_features, storage_type) of
-        {'EXIT', _} ->
-            ok;
-        disc_only_copies ->
-            ok;
-        _ ->
-            mnesia:delete_table(caps_features)
-    end,
-    mnesia:create_table(caps_features,
-                        [{disc_only_copies, [node()]},
-                         {local_content, true},
-                         {attributes,
-                          record_info(fields, caps_features)}]),
-    update_table(),
-    mnesia:add_table_copy(caps_features, node(),
-                          disc_only_copies);
-init_db(_, _) ->
-    ok.
+depends(_Host, _Opts) ->
+    [].
 
 init([Host, Opts]) ->
-    init_db(gen_mod:db_type(Host, Opts), Host),
+    Mod = gen_mod:db_mod(Host, Opts, ?MODULE),
+    Mod:init(Host, Opts),
     MaxSize = gen_mod:get_opt(cache_size, Opts,
                               fun(I) when is_integer(I), I>0 -> I end,
                               1000),
@@ -434,7 +424,7 @@ feature_response(#iq{type = result,
 	  Features = lists:flatmap(fun (#xmlel{name =
 						   <<"feature">>,
 					       attrs = FAttrs}) ->
-					   [xml:get_attr_s(<<"var">>, FAttrs)];
+					   [fxml:get_attr_s(<<"var">>, FAttrs)];
 				       (_) -> []
 				   end,
 				   Els),
@@ -450,65 +440,13 @@ feature_response(_IQResult, Host, From, Caps,
 
 caps_read_fun(Host, Node) ->
     LServer = jid:nameprep(Host),
-    DBType = gen_mod:db_type(LServer, ?MODULE),
-    caps_read_fun(LServer, Node, DBType).
-
-caps_read_fun(_LServer, Node, mnesia) ->
-    fun () ->
-	    case mnesia:dirty_read({caps_features, Node}) of
-	      [#caps_features{features = Features}] -> {ok, Features};
-	      _ -> error
-	    end
-    end;
-caps_read_fun(_LServer, Node, riak) ->
-    fun() ->
-            case ejabberd_riak:get(caps_features, caps_features_schema(), Node) of
-                {ok, #caps_features{features = Features}} -> {ok, Features};
-                _ -> error
-            end
-    end;
-caps_read_fun(LServer, {Node, SubNode}, odbc) ->
-    fun() ->
-            SNode = ejabberd_odbc:escape(Node),
-            SSubNode = ejabberd_odbc:escape(SubNode),
-            case ejabberd_odbc:sql_query(
-                   LServer, [<<"select feature from caps_features where ">>,
-                             <<"node='">>, SNode, <<"' and subnode='">>,
-                             SSubNode, <<"';">>]) of
-                {selected, [<<"feature">>], [[H]|_] = Fs} ->
-                    case catch jlib:binary_to_integer(H) of
-                        Int when is_integer(Int), Int>=0 ->
-                            {ok, Int};
-                        _ ->
-                            {ok, lists:flatten(Fs)}
-                    end;
-                _ ->
-                    error
-            end
-    end.
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    fun() -> Mod:caps_read(LServer, Node) end.
 
 caps_write_fun(Host, Node, Features) ->
     LServer = jid:nameprep(Host),
-    DBType = gen_mod:db_type(LServer, ?MODULE),
-    caps_write_fun(LServer, Node, Features, DBType).
-
-caps_write_fun(_LServer, Node, Features, mnesia) ->
-    fun () ->
-	    mnesia:dirty_write(#caps_features{node_pair = Node,
-					      features = Features})
-    end;
-caps_write_fun(_LServer, Node, Features, riak) ->
-    fun () ->
-            ejabberd_riak:put(#caps_features{node_pair = Node,
-                                             features = Features},
-			      caps_features_schema())
-    end;
-caps_write_fun(LServer, NodePair, Features, odbc) ->
-    fun () ->
-            ejabberd_odbc:sql_transaction(
-              LServer,
-              sql_write_features_t(NodePair, Features))
-    end.
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    fun() -> Mod:caps_write(LServer, Node, Features) end.
 
 make_my_disco_hash(Host) ->
     JID = jid:make(<<"">>, Host, <<"">>),
@@ -567,7 +505,7 @@ concat_features(Els) ->
     lists:usort(lists:flatmap(fun (#xmlel{name =
 					      <<"feature">>,
 					  attrs = Attrs}) ->
-				      [[xml:get_attr_s(<<"var">>, Attrs), $<]];
+				      [[fxml:get_attr_s(<<"var">>, Attrs), $<]];
 				  (_) -> []
 			      end,
 			      Els)).
@@ -576,11 +514,11 @@ concat_identities(Els) ->
     lists:sort(lists:flatmap(fun (#xmlel{name =
 					     <<"identity">>,
 					 attrs = Attrs}) ->
-				     [[xml:get_attr_s(<<"category">>, Attrs),
-				       $/, xml:get_attr_s(<<"type">>, Attrs),
+				     [[fxml:get_attr_s(<<"category">>, Attrs),
+				       $/, fxml:get_attr_s(<<"type">>, Attrs),
 				       $/,
-				       xml:get_attr_s(<<"xml:lang">>, Attrs),
-				       $/, xml:get_attr_s(<<"name">>, Attrs),
+				       fxml:get_attr_s(<<"xml:lang">>, Attrs),
+				       $/, fxml:get_attr_s(<<"name">>, Attrs),
 				       $<]];
 				 (_) -> []
 			     end,
@@ -589,8 +527,8 @@ concat_identities(Els) ->
 concat_info(Els) ->
     lists:sort(lists:flatmap(fun (#xmlel{name = <<"x">>,
 					 attrs = Attrs, children = Fields}) ->
-				     case {xml:get_attr_s(<<"xmlns">>, Attrs),
-					   xml:get_attr_s(<<"type">>, Attrs)}
+				     case {fxml:get_attr_s(<<"xmlns">>, Attrs),
+					   fxml:get_attr_s(<<"type">>, Attrs)}
 					 of
 				       {?NS_XDATA, <<"result">>} ->
 					   [concat_xdata_fields(Fields)];
@@ -606,10 +544,10 @@ concat_xdata_fields(Fields) ->
 					  attrs = Attrs, children = Els} =
 				       El,
 				   [FormType, VarFields] = Acc) ->
-				      case xml:get_attr_s(<<"var">>, Attrs) of
+				      case fxml:get_attr_s(<<"var">>, Attrs) of
 					<<"">> -> Acc;
 					<<"FORM_TYPE">> ->
-					    [xml:get_subtag_cdata(El,
+					    [fxml:get_subtag_cdata(El,
 								  <<"value">>),
 					     VarFields];
 					Var ->
@@ -622,7 +560,7 @@ concat_xdata_fields(Fields) ->
 										  children
 										      =
 										      VEls}) ->
-									      [[xml:get_cdata(VEls),
+									      [[fxml:get_cdata(VEls),
 										$<]];
 									  (_) ->
 									      []
@@ -658,64 +596,23 @@ is_valid_node(Node) ->
             false
     end.
 
-update_table() ->
-    Fields = record_info(fields, caps_features),
-    case mnesia:table_info(caps_features, attributes) of
-        Fields ->
-            ejabberd_config:convert_table_to_binary(
-              caps_features, Fields, set,
-              fun(#caps_features{node_pair = {N, _}}) -> N end,
-              fun(#caps_features{node_pair = {N, P},
-                                 features = Fs} = R) ->
-                      NewFs = if is_integer(Fs) ->
-                                      Fs;
-                                 true ->
-                                      [iolist_to_binary(F) || F <- Fs]
-                              end,
-                      R#caps_features{node_pair = {iolist_to_binary(N),
-                                                   iolist_to_binary(P)},
-                                      features = NewFs}
-              end);
-        _ ->
-            ?INFO_MSG("Recreating caps_features table", []),
-            mnesia:transform_table(caps_features, ignore, Fields)
-    end.
-
-sql_write_features_t({Node, SubNode}, Features) ->
-    SNode = ejabberd_odbc:escape(Node),
-    SSubNode = ejabberd_odbc:escape(SubNode),
-    NewFeatures = if is_integer(Features) ->
-                          [jlib:integer_to_binary(Features)];
-                     true ->
-                          Features
-                  end,
-    [[<<"delete from caps_features where node='">>,
-      SNode, <<"' and subnode='">>, SSubNode, <<"';">>]|
-     [[<<"insert into caps_features(node, subnode, feature) ">>,
-       <<"values ('">>, SNode, <<"', '">>, SSubNode, <<"', '">>,
-       ejabberd_odbc:escape(F), <<"');">>] || F <- NewFeatures]].
-
 caps_features_schema() ->
     {record_info(fields, caps_features), #caps_features{}}.
 
-export(_Server) ->
-    [{caps_features,
-      fun(_Host, #caps_features{node_pair = NodePair,
-                                features = Features}) ->
-              sql_write_features_t(NodePair, Features);
-         (_Host, _R) ->
-              []
-      end}].
+export(LServer) ->
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    Mod:export(LServer).
 
 import_info() ->
     [{<<"caps_features">>, 4}].
 
 import_start(LServer, DBType) ->
     ets:new(caps_features_tmp, [private, named_table, bag]),
-    init_db(DBType, LServer),
+    Mod = gen_mod:db_mod(DBType, ?MODULE),
+    Mod:init(LServer, []),
     ok.
 
-import(_LServer, {odbc, _}, _DBType, <<"caps_features">>,
+import(_LServer, {sql, _}, _DBType, <<"caps_features">>,
        [Node, SubNode, Feature, _TimeStamp]) ->
     Feature1 = case catch jlib:binary_to_integer(Feature) of
                    I when is_integer(I), I>0 -> I;
@@ -748,7 +645,7 @@ import_next(LServer, DBType, NodePair) ->
             ejabberd_riak:put(
               #caps_features{node_pair = NodePair, features = Features},
 	      caps_features_schema());
-        _ when DBType == odbc ->
+        _ when DBType == sql ->
             ok
     end,
     import_next(LServer, DBType, ets:next(caps_features_tmp, NodePair)).
@@ -757,6 +654,6 @@ mod_opt_type(cache_life_time) ->
     fun (I) when is_integer(I), I > 0 -> I end;
 mod_opt_type(cache_size) ->
     fun (I) when is_integer(I), I > 0 -> I end;
-mod_opt_type(db_type) -> fun gen_mod:v_db/1;
+mod_opt_type(db_type) -> fun(T) -> ejabberd_config:v_db(?MODULE, T) end;
 mod_opt_type(_) ->
     [cache_life_time, cache_size, db_type].
