@@ -5,7 +5,7 @@
 %%% Created : 15 Feb 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -33,11 +33,12 @@
 
 %% API
 -export([start_link/2, start/2, stop/1, export/1, import/1,
-	 import/3, closed_connection/3, get_connection_params/3]).
+	 import/3, closed_connection/3, get_connection_params/3,
+	 data_to_binary/2]).
 
 -export([init/1, handle_call/3, handle_cast/2,
 	 handle_info/2, terminate/2, code_change/3,
-	 mod_opt_type/1]).
+	 mod_opt_type/1, depends/2]).
 
 -include("ejabberd.hrl").
 -include("logger.hrl").
@@ -46,6 +47,8 @@
 
 -include("adhoc.hrl").
 
+-include("mod_irc.hrl").
+
 -define(DEFAULT_IRC_ENCODING, <<"iso8859-15">>).
 
 -define(DEFAULT_IRC_PORT, 6667).
@@ -58,27 +61,19 @@
 	[<<"koi8-r">>, <<"iso8859-15">>, <<"iso8859-1">>, <<"iso8859-2">>,
 	 <<"utf-8">>, <<"utf-8+latin-1">>]).
 
--type conn_param() :: {binary(), binary(), inet:port_number(), binary()} |
-                      {binary(), binary(), inet:port_number()} |
-                      {binary(), binary()} |
-                      {binary()}.
-
--record(irc_connection,
-        {jid_server_host = {#jid{}, <<"">>, <<"">>} :: {jid(), binary(), binary()},
-         pid = self()                               :: pid()}).
-
--record(irc_custom,
-        {us_host = {{<<"">>, <<"">>}, <<"">>} :: {{binary(), binary()},
-                                                  binary()},
-         data = [] :: [{username, binary()} |
-                       {connections_params, [conn_param()]}]}).
-
 -record(state, {host = <<"">>        :: binary(),
                 server_host = <<"">> :: binary(),
                 access = all         :: atom()}).
 
 -define(PROCNAME, ejabberd_mod_irc).
 
+-callback init(binary(), gen_mod:opts()) -> any().
+-callback import(binary(), #irc_custom{}) -> ok | pass.
+-callback get_data(binary(), binary(), {binary(), binary()}) ->
+    error | empty | irc_data().
+-callback set_data(binary(), binary(), {binary(), binary()}, irc_data()) ->
+    {atomic, any()}.
+
 %%====================================================================
 %% API
 %%====================================================================
@@ -95,7 +90,7 @@ start(Host, Opts) ->
     start_supervisor(Host),
     Proc = gen_mod:get_module_proc(Host, ?PROCNAME),
     ChildSpec = {Proc, {?MODULE, start_link, [Host, Opts]},
-		 temporary, 1000, worker, [?MODULE]},
+		 transient, 1000, worker, [?MODULE]},
     supervisor:start_child(ejabberd_sup, ChildSpec).
 
 stop(Host) ->
@@ -104,6 +99,9 @@ stop(Host) ->
     gen_server:call(Proc, stop),
     supervisor:delete_child(ejabberd_sup, Proc).
 
+depends(_Host, _Opts) ->
+    [].
+
 %%====================================================================
 %% gen_server callbacks
 %%====================================================================
@@ -116,24 +114,18 @@ stop(Host) ->
 %% Description: Initiates the server
 %%--------------------------------------------------------------------
 init([Host, Opts]) ->
-    ejabberd:start_app(p1_iconv),
+    ejabberd:start_app(iconv),
     MyHost = gen_mod:get_opt_host(Host, Opts,
 				  <<"irc.@HOST@">>),
-    case gen_mod:db_type(Host, Opts) of
-      mnesia ->
-	  mnesia:create_table(irc_custom,
-			      [{disc_copies, [node()]},
-			       {attributes, record_info(fields, irc_custom)}]),
-	  update_table();
-      _ -> ok
-    end,
+    Mod = gen_mod:db_mod(Host, Opts, ?MODULE),
+    Mod:init(Host, Opts),
     Access = gen_mod:get_opt(access, Opts,
-                             fun(A) when is_atom(A) -> A end,
+                             fun acl:access_rules_validator/1,
                              all),
     catch ets:new(irc_connection,
 		  [named_table, public,
 		   {keypos, #irc_connection.jid_server_host}]),
-    ejabberd_router:register_route(MyHost),
+    ejabberd_router:register_route(MyHost, Host),
     {ok,
      #state{host = MyHost, server_host = Host,
 	    access = Access}}.
@@ -216,7 +208,7 @@ do_route(Host, ServerHost, Access, From, To, Packet) ->
       allow -> do_route1(Host, ServerHost, From, To, Packet);
       _ ->
 	  #xmlel{attrs = Attrs} = Packet,
-	  Lang = xml:get_attr_s(<<"xml:lang">>, Attrs),
+	  Lang = fxml:get_attr_s(<<"xml:lang">>, Attrs),
 	  ErrText = <<"Access denied by service policy">>,
 	  Err = jlib:make_error_reply(Packet,
 				      ?ERRT_FORBIDDEN(Lang, ErrText)),
@@ -234,7 +226,7 @@ do_route1(Host, ServerHost, From, To, Packet) ->
 		  #iq{type = get, xmlns = (?NS_DISCO_INFO) = XMLNS,
 		      sub_el = SubEl, lang = Lang} =
 		      IQ ->
-		      Node = xml:get_tag_attr_s(<<"node">>, SubEl),
+		      Node = fxml:get_tag_attr_s(<<"node">>, SubEl),
 		      Info = ejabberd_hooks:run_fold(disco_info, ServerHost,
 						     [],
 						     [ServerHost, ?MODULE,
@@ -262,7 +254,7 @@ do_route1(Host, ServerHost, From, To, Packet) ->
 		  #iq{type = get, xmlns = (?NS_DISCO_ITEMS) = XMLNS,
 		      sub_el = SubEl, lang = Lang} =
 		      IQ ->
-		      Node = xml:get_tag_attr_s(<<"node">>, SubEl),
+		      Node = fxml:get_tag_attr_s(<<"node">>, SubEl),
 		      case Node of
 			<<>> ->
 			    ResIQ = IQ#iq{type = result,
@@ -304,8 +296,9 @@ do_route1(Host, ServerHost, From, To, Packet) ->
 									Lang)}]},
 			    Res = jlib:iq_to_xml(ResIQ);
 			_ ->
-			    Res = jlib:make_error_reply(Packet,
-							?ERR_ITEM_NOT_FOUND)
+			    Txt = <<"Node not found">>,
+			    Res = jlib:make_error_reply(
+				    Packet, ?ERRT_ITEM_NOT_FOUND(Lang, Txt))
 		      end,
 		      ejabberd_router:route(To, From, Res);
 		  #iq{xmlns = ?NS_REGISTER} = IQ ->
@@ -319,7 +312,7 @@ do_route1(Host, ServerHost, From, To, Packet) ->
 					      attrs = [{<<"xmlns">>, XMLNS}],
 					      children = iq_get_vcard(Lang)}]},
 		      ejabberd_router:route(To, From, jlib:iq_to_xml(Res));
-		  #iq{type = set, xmlns = ?NS_COMMANDS, lang = _Lang,
+		  #iq{type = set, xmlns = ?NS_COMMANDS, lang = Lang,
 		      sub_el = SubEl} =
 		      IQ ->
 		      Request = adhoc:parse_request(IQ),
@@ -348,8 +341,9 @@ do_route1(Host, ServerHost, From, To, Packet) ->
 			       true -> ok
 			    end;
 			_ ->
-			    Err = jlib:make_error_reply(Packet,
-							?ERR_ITEM_NOT_FOUND),
+			    Txt = <<"Node not found">>,
+			    Err = jlib:make_error_reply(
+				    Packet, ?ERRT_ITEM_NOT_FOUND(Lang, Txt)),
 			    ejabberd_router:route(To, From, Err)
 		      end;
 		  #iq{} = _IQ ->
@@ -407,12 +401,14 @@ do_route1(Host, ServerHost, From, To, Packet) ->
 		      ok
 		end;
 	    _ ->
+		Lang = fxml:get_tag_attr_s(<<"xml:lang">>, Packet),
 		case str:tokens(ChanServ, <<"!">>) of
 		  [<<_, _/binary>> = Nick, <<_, _/binary>> = Server] ->
 		      case ets:lookup(irc_connection, {From, Server, Host}) of
 			[] ->
-			    Err = jlib:make_error_reply(Packet,
-							?ERR_SERVICE_UNAVAILABLE),
+			    Txt = <<"IRC connection not found">>,
+			    Err = jlib:make_error_reply(
+				    Packet, ?ERRT_SERVICE_UNAVAILABLE(Lang, Txt)),
 			    ejabberd_router:route(To, From, Err);
 			[R] ->
 			    Pid = R#irc_connection.pid,
@@ -421,7 +417,9 @@ do_route1(Host, ServerHost, From, To, Packet) ->
 			    ok
 		      end;
 		  _ ->
-		      Err = jlib:make_error_reply(Packet, ?ERR_BAD_REQUEST),
+		      Txt = <<"Failed to parse chanserv">>,
+		      Err = jlib:make_error_reply(
+			      Packet, ?ERRT_BAD_REQUEST(Lang, Txt)),
 		      ejabberd_router:route(To, From, Err)
 		end
 	  end
@@ -474,7 +472,7 @@ iq_get_vcard(Lang) ->
 		[{xmlcdata,
 		  <<(translate:translate(Lang,
 					 <<"ejabberd IRC module">>))/binary,
-		    "\nCopyright (c) 2003-2015 ProcessOne">>}]}].
+		    "\nCopyright (c) 2003-2016 ProcessOne">>}]}].
 
 command_items(ServerHost, Host, Lang) ->
     lists:map(fun ({Node, Name, _Function}) ->
@@ -516,7 +514,7 @@ find_xdata_el1([]) -> false;
 find_xdata_el1([#xmlel{name = Name, attrs = Attrs,
 		       children = SubEls}
 		| Els]) ->
-    case xml:get_attr_s(<<"xmlns">>, Attrs) of
+    case fxml:get_attr_s(<<"xmlns">>, Attrs) of
       ?NS_XDATA ->
 	  #xmlel{name = Name, attrs = Attrs, children = SubEls};
       _ -> find_xdata_el1(Els)
@@ -532,10 +530,11 @@ process_irc_register(ServerHost, Host, From, _To,
 	  XDataEl = find_xdata_el(SubEl),
 	  case XDataEl of
 	    false ->
+		Txt1 = <<"No data form found">>,
 		IQ#iq{type = error,
-		      sub_el = [SubEl, ?ERR_NOT_ACCEPTABLE]};
+		      sub_el = [SubEl, ?ERRT_NOT_ACCEPTABLE(Lang, Txt1)]};
 	    #xmlel{attrs = Attrs} ->
-		case xml:get_attr_s(<<"type">>, Attrs) of
+		case fxml:get_attr_s(<<"type">>, Attrs) of
 		  <<"cancel">> ->
 		      IQ#iq{type = result,
 			    sub_el =
@@ -546,10 +545,11 @@ process_irc_register(ServerHost, Host, From, _To,
 		      XData = jlib:parse_xdata_submit(XDataEl),
 		      case XData of
 			invalid ->
+			    Txt2 = <<"Incorrect data form">>,
 			    IQ#iq{type = error,
-				  sub_el = [SubEl, ?ERR_BAD_REQUEST]};
+				  sub_el = [SubEl, ?ERRT_BAD_REQUEST(Lang, Txt2)]};
 			_ ->
-			    Node = str:tokens(xml:get_tag_attr_s(<<"node">>,
+			    Node = str:tokens(fxml:get_tag_attr_s(<<"node">>,
 								 SubEl),
 					      <<"/">>),
 			    case set_form(ServerHost, Host, From, Node, Lang,
@@ -567,11 +567,13 @@ process_irc_register(ServerHost, Host, From, _To,
 			    end
 		      end;
 		  _ ->
-		      IQ#iq{type = error, sub_el = [SubEl, ?ERR_BAD_REQUEST]}
+		      Txt3 = <<"Incorrect value of 'type' attribute">>,
+		      IQ#iq{type = error,
+			    sub_el = [SubEl, ?ERRT_BAD_REQUEST(Lang, Txt3)]}
 		end
 	  end;
       get ->
-	  Node = str:tokens(xml:get_tag_attr_s(<<"node">>, SubEl),
+	  Node = str:tokens(fxml:get_tag_attr_s(<<"node">>, SubEl),
 			    <<"/">>),
 	  case get_form(ServerHost, Host, From, Node, Lang) of
 	    {result, Res} ->
@@ -587,49 +589,16 @@ process_irc_register(ServerHost, Host, From, _To,
 
 get_data(ServerHost, Host, From) ->
     LServer = jid:nameprep(ServerHost),
-    get_data(LServer, Host, From,
-             gen_mod:db_type(LServer, ?MODULE)).
-
-get_data(_LServer, Host, From, mnesia) ->
-    #jid{luser = LUser, lserver = LServer} = From,
-    US = {LUser, LServer},
-    case catch mnesia:dirty_read({irc_custom, {US, Host}})
-	of
-      {'EXIT', _Reason} -> error;
-      [] -> empty;
-      [#irc_custom{data = Data}] -> Data
-    end;
-get_data(LServer, Host, From, riak) ->
-    #jid{luser = LUser, lserver = LServer} = From,
-    US = {LUser, LServer},
-    case ejabberd_riak:get(irc_custom, irc_custom_schema(), {US, Host}) of
-        {ok, #irc_custom{data = Data}} ->
-            Data;
-        {error, notfound} ->
-            empty;
-        _Err ->
-            error
-    end;
-get_data(LServer, Host, From, odbc) ->
-    SJID =
-	ejabberd_odbc:escape(jid:to_string(jid:tolower(jid:remove_resource(From)))),
-    SHost = ejabberd_odbc:escape(Host),
-    case catch ejabberd_odbc:sql_query(LServer,
-				       [<<"select data from irc_custom where jid='">>,
-					SJID, <<"' and host='">>, SHost,
-					<<"';">>])
-	of
-      {selected, [<<"data">>], [[SData]]} ->
-	  data_to_binary(From, ejabberd_odbc:decode_term(SData));
-      {'EXIT', _} -> error;
-      {selected, _, _} -> empty
-    end.
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    Mod:get_data(LServer, Host, From).
 
 get_form(ServerHost, Host, From, [], Lang) ->
     #jid{user = User, server = Server} = From,
     DefaultEncoding = get_default_encoding(Host),
     Customs = case get_data(ServerHost, Host, From) of
-		error -> {error, ?ERR_INTERNAL_SERVER_ERROR};
+		error ->
+		      Txt1 = <<"Database failure">>,
+		      {error, ?ERRT_INTERNAL_SERVER_ERROR(Lang, Txt1)};
 		empty -> {User, []};
 		Data -> get_username_and_connection_params(Data)
 	      end,
@@ -731,39 +700,10 @@ get_form(_ServerHost, _Host, _, _, _Lang) ->
 
 set_data(ServerHost, Host, From, Data) ->
     LServer = jid:nameprep(ServerHost),
-    set_data(LServer, Host, From, data_to_binary(From, Data),
-	     gen_mod:db_type(LServer, ?MODULE)).
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    Mod:set_data(LServer, Host, From, data_to_binary(From, Data)).
 
-set_data(_LServer, Host, From, Data, mnesia) ->
-    {LUser, LServer, _} = jid:tolower(From),
-    US = {LUser, LServer},
-    F = fun () ->
-		mnesia:write(#irc_custom{us_host = {US, Host},
-					 data = Data})
-	end,
-    mnesia:transaction(F);
-set_data(LServer, Host, From, Data, riak) ->
-    {LUser, LServer, _} = jid:tolower(From),
-    US = {LUser, LServer},
-    {atomic, ejabberd_riak:put(#irc_custom{us_host = {US, Host},
-                                           data = Data},
-			       irc_custom_schema())};
-set_data(LServer, Host, From, Data, odbc) ->
-    SJID =
-	ejabberd_odbc:escape(jid:to_string(jid:tolower(jid:remove_resource(From)))),
-    SHost = ejabberd_odbc:escape(Host),
-    SData = ejabberd_odbc:encode_term(Data),
-    F = fun () ->
-		odbc_queries:update_t(<<"irc_custom">>,
-				      [<<"jid">>, <<"host">>, <<"data">>],
-				      [SJID, SHost, SData],
-				      [<<"jid='">>, SJID, <<"' and host='">>,
-				       SHost, <<"'">>]),
-		ok
-	end,
-    ejabberd_odbc:sql_transaction(LServer, F).
-
-set_form(ServerHost, Host, From, [], _Lang, XData) ->
+set_form(ServerHost, Host, From, [], Lang, XData) ->
     case {lists:keysearch(<<"username">>, 1, XData),
 	  lists:keysearch(<<"connections_params">>, 1, XData)}
 	of
@@ -781,11 +721,11 @@ set_form(ServerHost, Host, From, [], _Lang, XData) ->
 				     {connections_params, ConnectionsParams}])
 			  of
 			{atomic, _} -> {result, []};
-			_ -> {error, ?ERR_NOT_ACCEPTABLE}
+			_ -> {error, ?ERRT_NOT_ACCEPTABLE(Lang, <<"Database failure">>)}
 		      end;
-		  _ -> {error, ?ERR_NOT_ACCEPTABLE}
+		  _ -> {error, ?ERRT_NOT_ACCEPTABLE(Lang, <<"Parse error">>)}
 		end;
-	    _ -> {error, ?ERR_NOT_ACCEPTABLE}
+	    _ -> {error, ?ERRT_NOT_ACCEPTABLE(Lang, <<"Scan error">>)}
 	  end;
       _ -> {error, ?ERR_NOT_ACCEPTABLE}
     end;
@@ -909,7 +849,9 @@ adhoc_join(From, To,
 						  elements = [Form]});
        true ->
 	   case jlib:parse_xdata_submit(XData) of
-	     invalid -> {error, ?ERR_BAD_REQUEST};
+	     invalid ->
+		 Txt1 = <<"Incorrect data form">>,
+		 {error, ?ERRT_BAD_REQUEST(Lang, Txt1)};
 	     Fields ->
 		 Channel = case lists:keysearch(<<"channel">>, 1, Fields)
 			       of
@@ -998,7 +940,8 @@ adhoc_register(ServerHost, From, To,
        true ->
 	   case jlib:parse_xdata_submit(XData) of
 	     invalid ->
-		 Error = {error, ?ERR_BAD_REQUEST},
+		 Txt1 = <<"Incorrect data form">>,
+		 Error = {error, ?ERRT_BAD_REQUEST(Lang, Txt1)},
 		 Username = false,
 		 ConnectionsParams = false;
 	     Fields ->
@@ -1021,7 +964,9 @@ adhoc_register(ServerHost, From, To,
 	     {atomic, _} ->
 		 adhoc:produce_response(Request,
 					#adhoc_response{status = completed});
-	     _ -> {error, ?ERR_INTERNAL_SERVER_ERROR}
+	     _ ->
+		 Txt2 = <<"Database failure">>,
+		 {error, ?ERRT_INTERNAL_SERVER_ERROR(Lang, Txt2)}
 	   end;
        true ->
 	   Form = generate_adhoc_register_form(Lang, Username,
@@ -1297,70 +1242,21 @@ conn_params_to_list(Params) ->
                Port, binary_to_list(P)}
       end, Params).
 
-irc_custom_schema() ->
-    {record_info(fields, irc_custom), #irc_custom{}}.
+export(LServer) ->
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    Mod:export(LServer).
 
-update_table() ->
-    Fields = record_info(fields, irc_custom),
-    case mnesia:table_info(irc_custom, attributes) of
-      Fields ->
-          ejabberd_config:convert_table_to_binary(
-            irc_custom, Fields, set,
-            fun(#irc_custom{us_host = {_, H}}) -> H end,
-            fun(#irc_custom{us_host = {{U, S}, H},
-                            data = Data} = R) ->
-		    JID = jid:make(U, S, <<"">>),
-                    R#irc_custom{us_host = {{iolist_to_binary(U),
-                                             iolist_to_binary(S)},
-                                            iolist_to_binary(H)},
-                                 data = data_to_binary(JID, Data)}
-            end);
-      _ ->
-	  ?INFO_MSG("Recreating irc_custom table", []),
-	  mnesia:transform_table(irc_custom, ignore, Fields)
-    end.
+import(LServer) ->
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    Mod:import(LServer).
 
-export(_Server) ->
-    [{irc_custom,
-      fun(Host, #irc_custom{us_host = {{U, S}, IRCHost},
-                            data = Data}) ->
-              case str:suffix(Host, IRCHost) of
-                  true ->
-                      SJID = ejabberd_odbc:escape(
-                               jid:to_string(
-                                 jid:make(U, S, <<"">>))),
-                      SIRCHost = ejabberd_odbc:escape(IRCHost),
-                      SData = ejabberd_odbc:encode_term(Data),
-                      [[<<"delete from irc_custom where jid='">>, SJID,
-                        <<"' and host='">>, SIRCHost, <<"';">>],
-                       [<<"insert into irc_custom(jid, host, "
-                          "data) values ('">>,
-                        SJID, <<"', '">>, SIRCHost, <<"', '">>, SData,
-                        <<"');">>]];
-                  false ->
-                      []
-              end
-      end}].
-
-import(_LServer) ->
-    [{<<"select jid, host, data from irc_custom;">>,
-      fun([SJID, IRCHost, SData]) ->
-              #jid{luser = U, lserver = S} = jid:from_string(SJID),
-              Data = ejabberd_odbc:decode_term(SData),
-              #irc_custom{us_host = {{U, S}, IRCHost},
-                          data = Data}
-      end}].
-
-import(_LServer, mnesia, #irc_custom{} = R) ->
-    mnesia:dirty_write(R);
-import(_LServer, riak, #irc_custom{} = R) ->
-    ejabberd_riak:put(R, irc_custom_schema());
-import(_, _, _) ->
-    pass.
+import(LServer, DBType, Data) ->
+    Mod = gen_mod:db_mod(DBType, ?MODULE),
+    Mod:import(LServer, Data).
 
 mod_opt_type(access) ->
-    fun (A) when is_atom(A) -> A end;
-mod_opt_type(db_type) -> fun gen_mod:v_db/1;
+    fun acl:access_rules_validator/1;
+mod_opt_type(db_type) -> fun(T) -> ejabberd_config:v_db(?MODULE, T) end;
 mod_opt_type(default_encoding) ->
     fun iolist_to_binary/1;
 mod_opt_type(host) -> fun iolist_to_binary/1;
@@ -1368,7 +1264,7 @@ mod_opt_type(_) ->
     [access, db_type, default_encoding, host].
 
 extract_ident(Packet) ->
-    case xml:get_subtag(Packet, <<"headers">>) of
+    case fxml:get_subtag(Packet, <<"headers">>) of
 	{xmlel, _Name, _Attrs, Headers} ->
 	    extract_header(<<"X-Irc-Ident">>, Headers);
 	_ ->
@@ -1376,7 +1272,7 @@ extract_ident(Packet) ->
     end.
 
 extract_ip_address(Packet) ->
-    case xml:get_subtag(Packet, <<"headers">>) of
+    case fxml:get_subtag(Packet, <<"headers">>) of
 	{xmlel, _Name, _Attrs, Headers} ->
 	    extract_header(<<"X-Forwarded-For">>, Headers);
 	_ ->
@@ -1384,7 +1280,7 @@ extract_ip_address(Packet) ->
     end.
 
 extract_header(HeaderName, [{xmlel, _Name, _Attrs, [{xmlcdata, Value}]} | Tail]) ->
-    case xml:get_attr(<<"name">>, _Attrs) of
+    case fxml:get_attr(<<"name">>, _Attrs) of
 	{value, HeaderName} ->
 	    binary_to_list(Value);
 	_ ->
