@@ -5,7 +5,7 @@
 %%% Created : 19 Mar 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -48,29 +48,18 @@
 	 export/1,
 	 import/1,
 	 import/3,
+	 opts_to_binary/1,
 	 can_use_nick/4]).
 
 -export([init/1, handle_call/3, handle_cast/2,
 	 handle_info/2, terminate/2, code_change/3,
-	 mod_opt_type/1]).
+	 mod_opt_type/1, depends/2]).
 
 -include("ejabberd.hrl").
 -include("logger.hrl").
 
 -include("jlib.hrl").
-
--record(muc_room, {name_host = {<<"">>, <<"">>} :: {binary(), binary()} |
-                                                   {'_', binary()},
-                   opts = [] :: list() | '_'}).
-
--record(muc_online_room,
-        {name_host = {<<"">>, <<"">>} :: {binary(), binary()} | '$1' |
-                                         {'_', binary()} | '_',
-         pid = self() :: pid() | '$2' | '_' | '$1'}).
-
--record(muc_registered,
-        {us_host = {{<<"">>, <<"">>}, <<"">>} :: {{binary(), binary()}, binary()} | '$1',
-         nick = <<"">> :: binary()}).
+-include("mod_muc.hrl").
 
 -record(state,
 	{host = <<"">> :: binary(),
@@ -84,6 +73,17 @@
 
 -define(MAX_ROOMS_DISCOITEMS, 100).
 
+-type muc_room_opts() :: [{atom(), any()}].
+-callback init(binary(), gen_mod:opts()) -> any().
+-callback import(binary(), #muc_room{} | #muc_registered{}) -> ok | pass.
+-callback store_room(binary(), binary(), binary(), list()) -> {atomic, any()}.
+-callback restore_room(binary(), binary(), binary()) -> muc_room_opts() | error.
+-callback forget_room(binary(), binary(), binary()) -> {atomic, any()}.
+-callback can_use_nick(binary(), binary(), jid(), binary()) -> boolean().
+-callback get_rooms(binary(), binary()) -> [#muc_room{}].
+-callback get_nick(binary(), binary(), jid()) -> binary() | error.
+-callback set_nick(binary(), binary(), jid(), binary()) -> {atomic, ok | false}.
+
 %%====================================================================
 %% API
 %%====================================================================
@@ -95,7 +95,7 @@ start_link(Host, Opts) ->
 start(Host, Opts) ->
     Proc = gen_mod:get_module_proc(Host, ?PROCNAME),
     ChildSpec = {Proc, {?MODULE, start_link, [Host, Opts]},
-		 temporary, 1000, worker, [?MODULE]},
+		 transient, 1000, worker, [?MODULE]},
     supervisor:start_child(ejabberd_sup, ChildSpec).
 
 stop(Host) ->
@@ -105,6 +105,9 @@ stop(Host) ->
     supervisor:delete_child(ejabberd_sup, Proc),
     {wait, Rooms}.
 
+depends(_Host, _Opts) ->
+    [{mod_mam, soft}].
+
 shutdown_rooms(Host) ->
     MyHost = gen_mod:get_module_opt_host(Host, mod_muc,
 					 <<"conference.@HOST@">>),
@@ -137,104 +140,24 @@ create_room(Host, Name, From, Nick, Opts) ->
 
 store_room(ServerHost, Host, Name, Opts) ->
     LServer = jid:nameprep(ServerHost),
-    store_room(LServer, Host, Name, Opts,
-	       gen_mod:db_type(LServer, ?MODULE)).
-
-store_room(_LServer, Host, Name, Opts, mnesia) ->
-    F = fun () ->
-		mnesia:write(#muc_room{name_host = {Name, Host},
-				       opts = Opts})
-	end,
-    mnesia:transaction(F);
-store_room(_LServer, Host, Name, Opts, riak) ->
-    {atomic, ejabberd_riak:put(#muc_room{name_host = {Name, Host},
-                                         opts = Opts},
-			       muc_room_schema())};
-store_room(LServer, Host, Name, Opts, odbc) ->
-    SName = ejabberd_odbc:escape(Name),
-    SHost = ejabberd_odbc:escape(Host),
-    SOpts = ejabberd_odbc:encode_term(Opts),
-    F = fun () ->
-		odbc_queries:update_t(<<"muc_room">>,
-				      [<<"name">>, <<"host">>, <<"opts">>],
-				      [SName, SHost, SOpts],
-				      [<<"name='">>, SName, <<"' and host='">>,
-				       SHost, <<"'">>])
-	end,
-    ejabberd_odbc:sql_transaction(LServer, F).
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    Mod:store_room(LServer, Host, Name, Opts).
 
 restore_room(ServerHost, Host, Name) ->
     LServer = jid:nameprep(ServerHost),
-    restore_room(LServer, Host, Name,
-                 gen_mod:db_type(LServer, ?MODULE)).
-
-restore_room(_LServer, Host, Name, mnesia) ->
-    case catch mnesia:dirty_read(muc_room, {Name, Host}) of
-      [#muc_room{opts = Opts}] -> Opts;
-      _ -> error
-    end;
-restore_room(_LServer, Host, Name, riak) ->
-    case ejabberd_riak:get(muc_room, muc_room_schema(), {Name, Host}) of
-        {ok, #muc_room{opts = Opts}} -> Opts;
-        _ -> error
-    end;
-restore_room(LServer, Host, Name, odbc) ->
-    SName = ejabberd_odbc:escape(Name),
-    SHost = ejabberd_odbc:escape(Host),
-    case catch ejabberd_odbc:sql_query(LServer,
-				       [<<"select opts from muc_room where name='">>,
-					SName, <<"' and host='">>, SHost,
-					<<"';">>])
-	of
-      {selected, [<<"opts">>], [[Opts]]} ->
-	  opts_to_binary(ejabberd_odbc:decode_term(Opts));
-      _ -> error
-    end.
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    Mod:restore_room(LServer, Host, Name).
 
 forget_room(ServerHost, Host, Name) ->
     LServer = jid:nameprep(ServerHost),
-    forget_room(LServer, Host, Name,
-		gen_mod:db_type(LServer, ?MODULE)).
-
-forget_room(LServer, Host, Name, mnesia) ->
-    remove_room_mam(LServer, Host, Name),
-    F = fun () -> mnesia:delete({muc_room, {Name, Host}})
-	end,
-    mnesia:transaction(F);
-forget_room(LServer, Host, Name, riak) ->
-    remove_room_mam(LServer, Host, Name),
-    {atomic, ejabberd_riak:delete(muc_room, {Name, Host})};
-forget_room(LServer, Host, Name, odbc) ->
-    remove_room_mam(LServer, Host, Name),
-    SName = ejabberd_odbc:escape(Name),
-    SHost = ejabberd_odbc:escape(Host),
-    F = fun () ->
-		ejabberd_odbc:sql_query_t([<<"delete from muc_room where name='">>,
-					   SName, <<"' and host='">>, SHost,
-					   <<"';">>])
-	end,
-    ejabberd_odbc:sql_transaction(LServer, F).
-
-remove_room_mam(LServer, Host, Name) ->
-    case gen_mod:is_loaded(LServer, mod_mam) of
-	true ->
-	    U = jid:nodeprep(Name),
-	    S = jid:nameprep(Host),
-	    DBType = gen_mod:db_type(LServer, mod_mam),
-	    if DBType == odbc ->
-		    mod_mam:remove_user(jid:to_string({U, S, <<>>}),
-					LServer, DBType);
-	       true ->
-		    mod_mam:remove_user(U, S, DBType)
-	    end;
-	false ->
-	    ok
-    end.
+    ejabberd_hooks:run(remove_room, LServer, [LServer, Name, Host]),
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    Mod:forget_room(LServer, Host, Name).
 
 process_iq_disco_items(Host, From, To,
 		       #iq{lang = Lang} = IQ) ->
     Rsm = jlib:rsm_decode(IQ),
-    DiscoNode = xml:get_tag_attr_s(<<"node">>, IQ#iq.sub_el),
+    DiscoNode = fxml:get_tag_attr_s(<<"node">>, IQ#iq.sub_el),
     Res = IQ#iq{type = result,
 		sub_el =
 		    [#xmlel{name = <<"query">>,
@@ -245,48 +168,8 @@ process_iq_disco_items(Host, From, To,
 can_use_nick(_ServerHost, _Host, _JID, <<"">>) -> false;
 can_use_nick(ServerHost, Host, JID, Nick) ->
     LServer = jid:nameprep(ServerHost),
-    can_use_nick(LServer, Host, JID, Nick,
-		 gen_mod:db_type(LServer, ?MODULE)).
-
-can_use_nick(_LServer, Host, JID, Nick, mnesia) ->
-    {LUser, LServer, _} = jid:tolower(JID),
-    LUS = {LUser, LServer},
-    case catch mnesia:dirty_select(muc_registered,
-				   [{#muc_registered{us_host = '$1',
-						     nick = Nick, _ = '_'},
-				     [{'==', {element, 2, '$1'}, Host}],
-				     ['$_']}])
-	of
-      {'EXIT', _Reason} -> true;
-      [] -> true;
-      [#muc_registered{us_host = {U, _Host}}] -> U == LUS
-    end;
-can_use_nick(LServer, Host, JID, Nick, riak) ->
-    {LUser, LServer, _} = jid:tolower(JID),
-    LUS = {LUser, LServer},
-    case ejabberd_riak:get_by_index(muc_registered,
-				    muc_registered_schema(),
-                                    <<"nick_host">>, {Nick, Host}) of
-        {ok, []} ->
-            true;
-        {ok, [#muc_registered{us_host = {U, _Host}}]} ->
-            U == LUS;
-        {error, _} ->
-            true
-    end;
-can_use_nick(LServer, Host, JID, Nick, odbc) ->
-    SJID =
-	jid:to_string(jid:tolower(jid:remove_resource(JID))),
-    SNick = ejabberd_odbc:escape(Nick),
-    SHost = ejabberd_odbc:escape(Host),
-    case catch ejabberd_odbc:sql_query(LServer,
-				       [<<"select jid from muc_registered ">>,
-					<<"where nick='">>, SNick,
-					<<"' and host='">>, SHost, <<"';">>])
-	of
-      {selected, [<<"jid">>], [[SJID1]]} -> SJID == SJID1;
-      _ -> true
-    end.
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    Mod:can_use_nick(LServer, Host, JID, Nick).
 
 %%====================================================================
 %% gen_server callbacks
@@ -295,21 +178,8 @@ can_use_nick(LServer, Host, JID, Nick, odbc) ->
 init([Host, Opts]) ->
     MyHost = gen_mod:get_opt_host(Host, Opts,
 				  <<"conference.@HOST@">>),
-    case gen_mod:db_type(Host, Opts) of
-        mnesia ->
-            mnesia:create_table(muc_room,
-                                [{disc_copies, [node()]},
-                                 {attributes,
-                                  record_info(fields, muc_room)}]),
-            mnesia:create_table(muc_registered,
-                                [{disc_copies, [node()]},
-                                 {attributes,
-                                  record_info(fields, muc_registered)}]),
-            update_tables(MyHost),
-            mnesia:add_table_index(muc_registered, nick);
-        _ ->
-            ok
-    end,
+    Mod = gen_mod:db_mod(Host, Opts, ?MODULE),
+    Mod:init(Host, [{host, MyHost}|Opts]),
     mnesia:create_table(muc_online_room,
 			[{ram_copies, [node()]},
 			 {attributes, record_info(fields, muc_online_room)}]),
@@ -318,14 +188,14 @@ init([Host, Opts]) ->
     clean_table_from_bad_node(node(), MyHost),
     mnesia:subscribe(system),
     Access = gen_mod:get_opt(access, Opts,
-                             fun(A) when is_atom(A) -> A end, all),
+                             fun acl:access_rules_validator/1, all),
     AccessCreate = gen_mod:get_opt(access_create, Opts,
-                                   fun(A) when is_atom(A) -> A end, all),
+                                   fun acl:access_rules_validator/1, all),
     AccessAdmin = gen_mod:get_opt(access_admin, Opts,
-                                  fun(A) when is_atom(A) -> A end,
+                                  fun acl:access_rules_validator/1,
                                   none),
     AccessPersistent = gen_mod:get_opt(access_persistent, Opts,
-				       fun(A) when is_atom(A) -> A end,
+				       fun acl:access_rules_validator/1,
                                        all),
     HistorySize = gen_mod:get_opt(history_size, Opts,
                                   fun(I) when is_integer(I), I>=0 -> I end,
@@ -355,6 +225,7 @@ init([Host, Opts]) ->
 			     public -> Bool;
 			     public_list -> Bool;
 			     mam -> Bool;
+			     allow_subscription -> Bool;
 			     password -> fun iolist_to_binary/1;
 			     title -> fun iolist_to_binary/1;
 			     allow_private_messages_from_visitors ->
@@ -385,7 +256,7 @@ init([Host, Opts]) ->
     RoomShaper = gen_mod:get_opt(room_shaper, Opts,
                                  fun(A) when is_atom(A) -> A end,
                                  none),
-    ejabberd_router:register_route(MyHost),
+    ejabberd_router:register_route(MyHost, Host),
     load_permanent_rooms(MyHost, Host,
 			 {Access, AccessCreate, AccessAdmin, AccessPersistent},
 			 HistorySize, RoomShaper),
@@ -463,7 +334,7 @@ do_route(Host, ServerHost, Access, HistorySize, RoomShaper,
 		From, To, Packet, DefRoomOpts);
 	_ ->
 	    #xmlel{attrs = Attrs} = Packet,
-	    Lang = xml:get_attr_s(<<"xml:lang">>, Attrs),
+	    Lang = fxml:get_attr_s(<<"xml:lang">>, Attrs),
 	    ErrText = <<"Access denied by service policy">>,
 	    Err = jlib:make_error_reply(Packet,
 		    ?ERRT_FORBIDDEN(Lang, ErrText)),
@@ -551,6 +422,18 @@ do_route1(Host, ServerHost, Access, HistorySize, RoomShaper,
 							iq_get_vcard(Lang)}]},
 			    ejabberd_router:route(To, From,
 						  jlib:iq_to_xml(Res));
+			#iq{type = get, xmlns = ?NS_MUCSUB,
+			    sub_el = #xmlel{name = <<"subscriptions">>} = SubEl} = IQ ->
+			      RoomJIDs = get_subscribed_rooms(ServerHost, Host, From),
+			      Subs = lists:map(
+				       fun(J) ->
+					       #xmlel{name = <<"subscription">>,
+						      attrs = [{<<"jid">>,
+								jid:to_string(J)}]}
+				       end, RoomJIDs),
+			      Res = IQ#iq{type = result,
+					  sub_el = [SubEl#xmlel{children = Subs}]},
+			      ejabberd_router:route(To, From, jlib:iq_to_xml(Res));
 			#iq{type = get, xmlns = ?NS_MUC_UNIQUE} = IQ ->
 			    Res = IQ#iq{type = result,
 					sub_el =
@@ -569,18 +452,18 @@ do_route1(Host, ServerHost, Access, HistorySize, RoomShaper,
 			_ -> ok
 		      end;
 		  <<"message">> ->
-		      case xml:get_attr_s(<<"type">>, Attrs) of
+		      case fxml:get_attr_s(<<"type">>, Attrs) of
 			<<"error">> -> ok;
 			_ ->
 			    case acl:match_rule(ServerHost, AccessAdmin, From)
 				of
 			      allow ->
-				  Msg = xml:get_path_s(Packet,
+				  Msg = fxml:get_path_s(Packet,
 						       [{elem, <<"body">>},
 							cdata]),
 				  broadcast_service_message(Host, Msg);
 			      _ ->
-				  Lang = xml:get_attr_s(<<"xml:lang">>, Attrs),
+				  Lang = fxml:get_attr_s(<<"xml:lang">>, Attrs),
 				  ErrText =
 				      <<"Only service administrators are allowed "
 					"to send service messages">>,
@@ -593,7 +476,7 @@ do_route1(Host, ServerHost, Access, HistorySize, RoomShaper,
 		  <<"presence">> -> ok
 		end;
 	    _ ->
-		case xml:get_attr_s(<<"type">>, Attrs) of
+		case fxml:get_attr_s(<<"type">>, Attrs) of
 		  <<"error">> -> ok;
 		  <<"result">> -> ok;
 		  _ ->
@@ -605,11 +488,12 @@ do_route1(Host, ServerHost, Access, HistorySize, RoomShaper,
       _ ->
 	    case mnesia:dirty_read(muc_online_room, {Room, Host}) of
 		[] ->
-		    Type = xml:get_attr_s(<<"type">>, Attrs),
+		    Type = fxml:get_attr_s(<<"type">>, Attrs),
 		    case {Name, Type} of
 			{<<"presence">>, <<"">>} ->
 			    case check_user_can_create_room(ServerHost,
-				    AccessCreate, From, Room) of
+				    AccessCreate, From, Room) and
+				check_create_roomid(ServerHost, Room) of
 				true ->
 				    {ok, Pid} = start_new_room(Host, ServerHost, Access,
 					    Room, HistorySize,
@@ -618,14 +502,14 @@ do_route1(Host, ServerHost, Access, HistorySize, RoomShaper,
 				    mod_muc_room:route(Pid, From, Nick, Packet),
 				    ok;
 				false ->
-				    Lang = xml:get_attr_s(<<"xml:lang">>, Attrs),
+				    Lang = fxml:get_attr_s(<<"xml:lang">>, Attrs),
 				    ErrText = <<"Room creation is denied by service policy">>,
 				    Err = jlib:make_error_reply(
 					    Packet, ?ERRT_FORBIDDEN(Lang, ErrText)),
 				    ejabberd_router:route(To, From, Err)
 			    end;
 			_ ->
-			    Lang = xml:get_attr_s(<<"xml:lang">>, Attrs),
+			    Lang = fxml:get_attr_s(<<"xml:lang">>, Attrs),
 			    ErrText = <<"Conference room does not exist">>,
 			    Err = jlib:make_error_reply(Packet,
 				    ?ERRT_ITEM_NOT_FOUND(Lang, ErrText)),
@@ -640,56 +524,26 @@ do_route1(Host, ServerHost, Access, HistorySize, RoomShaper,
     end.
 
 check_user_can_create_room(ServerHost, AccessCreate,
-			   From, RoomID) ->
+			   From, _RoomID) ->
     case acl:match_rule(ServerHost, AccessCreate, From) of
-      allow ->
-	  byte_size(RoomID) =<
-	    gen_mod:get_module_opt(ServerHost, ?MODULE, max_room_id,
-                                   fun(infinity) -> infinity;
-                                      (I) when is_integer(I), I>0 -> I
-                                   end, infinity);
+      allow -> true;
       _ -> false
     end.
 
+check_create_roomid(ServerHost, RoomID) ->
+    Max = gen_mod:get_module_opt(ServerHost, ?MODULE, max_room_id,
+				 fun(infinity) -> infinity;
+				    (I) when is_integer(I), I>0 -> I
+				 end, infinity),
+    Regexp = gen_mod:get_module_opt(ServerHost, ?MODULE, regexp_room_id,
+				    fun iolist_to_binary/1, ""),
+    (byte_size(RoomID) =< Max) and
+    (re:run(RoomID, Regexp, [unicode, {capture, none}]) == match).
+
 get_rooms(ServerHost, Host) ->
     LServer = jid:nameprep(ServerHost),
-    get_rooms(LServer, Host,
-              gen_mod:db_type(LServer, ?MODULE)).
-
-get_rooms(_LServer, Host, mnesia) ->
-    case catch mnesia:dirty_select(muc_room,
-				   [{#muc_room{name_host = {'_', Host},
-					       _ = '_'},
-				     [], ['$_']}])
-	of
-      {'EXIT', Reason} -> ?ERROR_MSG("~p", [Reason]), [];
-      Rs -> Rs
-    end;
-get_rooms(_LServer, Host, riak) ->
-    case ejabberd_riak:get(muc_room, muc_room_schema()) of
-        {ok, Rs} ->
-            lists:filter(
-              fun(#muc_room{name_host = {_, H}}) ->
-                      Host == H
-              end, Rs);
-        _Err ->
-            []
-    end;
-get_rooms(LServer, Host, odbc) ->
-    SHost = ejabberd_odbc:escape(Host),
-    case catch ejabberd_odbc:sql_query(LServer,
-				       [<<"select name, opts from muc_room ">>,
-					<<"where host='">>, SHost, <<"';">>])
-	of
-      {selected, [<<"name">>, <<"opts">>], RoomOpts} ->
-	  lists:map(fun ([Room, Opts]) ->
-			    #muc_room{name_host = {Room, Host},
-				      opts = opts_to_binary(
-                                               ejabberd_odbc:decode_term(Opts))}
-		    end,
-		    RoomOpts);
-      Err -> ?ERROR_MSG("failed to get rooms: ~p", [Err]), []
-    end.
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    Mod:get_rooms(LServer, Host).
 
 load_permanent_rooms(Host, ServerHost, Access,
 		     HistorySize, RoomShaper) ->
@@ -751,11 +605,15 @@ iq_disco_info(ServerHost, Lang) ->
 	    attrs = [{<<"var">>, ?NS_REGISTER}], children = []},
      #xmlel{name = <<"feature">>,
 	    attrs = [{<<"var">>, ?NS_RSM}], children = []},
+     #xmlel{name = <<"feature">>,
+	    attrs = [{<<"var">>, ?NS_MUCSUB}], children = []},
      #xmlel{name = <<"feature">>,
 	    attrs = [{<<"var">>, ?NS_VCARD}], children = []}] ++
 	case gen_mod:is_loaded(ServerHost, mod_mam) of
 	    true ->
 		[#xmlel{name = <<"feature">>,
+			attrs = [{<<"var">>, ?NS_MAM_TMP}]},
+		 #xmlel{name = <<"feature">>,
 			attrs = [{<<"var">>, ?NS_MAM_0}]},
 		 #xmlel{name = <<"feature">>,
 			attrs = [{<<"var">>, ?NS_MAM_1}]}];
@@ -845,6 +703,19 @@ get_vh_rooms(Host, #rsm_in{max=M, direction=Direction, id=I, index=Index})->
 		     index = NewIndex}}
     end.
 
+get_subscribed_rooms(ServerHost, Host, From) ->
+    Rooms = get_rooms(ServerHost, Host),
+    lists:flatmap(
+      fun(#muc_room{name_host = {Name, _}, opts = Opts}) ->
+	      Subscribers = proplists:get_value(subscribers, Opts, []),
+	      case lists:keymember(From, 1, Subscribers) of
+		  true -> [jid:make(Name, Host, <<>>)];
+		  false -> []
+	      end;
+	 (_) ->
+	      []
+      end, Rooms).
+
 %% @doc Return the position of desired room in the list of rooms.
 %% The room must exist in the list. The count starts in 0.
 %% @spec (Desired::muc_online_room(), Rooms::[muc_online_room()]) -> integer()
@@ -877,41 +748,8 @@ iq_get_unique(From) ->
 
 get_nick(ServerHost, Host, From) ->
     LServer = jid:nameprep(ServerHost),
-    get_nick(LServer, Host, From,
-	     gen_mod:db_type(LServer, ?MODULE)).
-
-get_nick(_LServer, Host, From, mnesia) ->
-    {LUser, LServer, _} = jid:tolower(From),
-    LUS = {LUser, LServer},
-    case catch mnesia:dirty_read(muc_registered,
-				 {LUS, Host})
-	of
-      {'EXIT', _Reason} -> error;
-      [] -> error;
-      [#muc_registered{nick = Nick}] -> Nick
-    end;
-get_nick(LServer, Host, From, riak) ->
-    {LUser, LServer, _} = jid:tolower(From),
-    US = {LUser, LServer},
-    case ejabberd_riak:get(muc_registered,
-			   muc_registered_schema(),
-			   {US, Host}) of
-        {ok, #muc_registered{nick = Nick}} -> Nick;
-        {error, _} -> error
-    end;
-get_nick(LServer, Host, From, odbc) ->
-    SJID =
-	ejabberd_odbc:escape(jid:to_string(jid:tolower(jid:remove_resource(From)))),
-    SHost = ejabberd_odbc:escape(Host),
-    case catch ejabberd_odbc:sql_query(LServer,
-				       [<<"select nick from muc_registered where "
-					  "jid='">>,
-					SJID, <<"' and host='">>, SHost,
-					<<"';">>])
-	of
-      {selected, [<<"nick">>], [[Nick]]} -> Nick;
-      _ -> error
-    end.
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    Mod:get_nick(LServer, Host, From).
 
 iq_get_register_info(ServerHost, Host, From, Lang) ->
     {Nick, Registered} = case get_nick(ServerHost, Host,
@@ -950,107 +788,8 @@ iq_get_register_info(ServerHost, Host, From, Lang) ->
 
 set_nick(ServerHost, Host, From, Nick) ->
     LServer = jid:nameprep(ServerHost),
-    set_nick(LServer, Host, From, Nick,
-	     gen_mod:db_type(LServer, ?MODULE)).
-
-set_nick(_LServer, Host, From, Nick, mnesia) ->
-    {LUser, LServer, _} = jid:tolower(From),
-    LUS = {LUser, LServer},
-    F = fun () ->
-		case Nick of
-		  <<"">> ->
-		      mnesia:delete({muc_registered, {LUS, Host}}), ok;
-		  _ ->
-		      Allow = case mnesia:select(muc_registered,
-						 [{#muc_registered{us_host =
-								       '$1',
-								   nick = Nick,
-								   _ = '_'},
-						   [{'==', {element, 2, '$1'},
-						     Host}],
-						   ['$_']}])
-				  of
-				[] -> true;
-				[#muc_registered{us_host = {U, _Host}}] ->
-				    U == LUS
-			      end,
-		      if Allow ->
-			     mnesia:write(#muc_registered{us_host = {LUS, Host},
-							  nick = Nick}),
-			     ok;
-			 true -> false
-		      end
-		end
-	end,
-    mnesia:transaction(F);
-set_nick(LServer, Host, From, Nick, riak) ->
-    {LUser, LServer, _} = jid:tolower(From),
-    LUS = {LUser, LServer},
-    {atomic,
-     case Nick of
-         <<"">> ->
-             ejabberd_riak:delete(muc_registered, {LUS, Host});
-         _ ->
-             Allow = case ejabberd_riak:get_by_index(
-                            muc_registered,
-			    muc_registered_schema(),
-                            <<"nick_host">>, {Nick, Host}) of
-                         {ok, []} ->
-                             true;
-                         {ok, [#muc_registered{us_host = {U, _Host}}]} ->
-                             U == LUS;
-                         {error, _} ->
-                             false
-                     end,
-             if Allow ->
-                     ejabberd_riak:put(#muc_registered{us_host = {LUS, Host},
-                                                       nick = Nick},
-				       muc_registered_schema(),
-                                       [{'2i', [{<<"nick_host">>,
-                                                 {Nick, Host}}]}]);
-                true ->
-                     false
-             end
-     end};
-set_nick(LServer, Host, From, Nick, odbc) ->
-    JID =
-	jid:to_string(jid:tolower(jid:remove_resource(From))),
-    SJID = ejabberd_odbc:escape(JID),
-    SNick = ejabberd_odbc:escape(Nick),
-    SHost = ejabberd_odbc:escape(Host),
-    F = fun () ->
-		case Nick of
-		  <<"">> ->
-		      ejabberd_odbc:sql_query_t([<<"delete from muc_registered where ">>,
-						 <<"jid='">>, SJID,
-						 <<"' and host='">>, Host,
-						 <<"';">>]),
-		      ok;
-		  _ ->
-		      Allow = case
-				ejabberd_odbc:sql_query_t([<<"select jid from muc_registered ">>,
-							   <<"where nick='">>,
-							   SNick,
-							   <<"' and host='">>,
-							   SHost, <<"';">>])
-				  of
-				{selected, [<<"jid">>], [[J]]} -> J == JID;
-				_ -> true
-			      end,
-		      if Allow ->
-			     odbc_queries:update_t(<<"muc_registered">>,
-						   [<<"jid">>, <<"host">>,
-						    <<"nick">>],
-						   [SJID, SHost, SNick],
-						   [<<"jid='">>, SJID,
-						    <<"' and host='">>, SHost,
-						    <<"'">>]),
-			     ok;
-			 true -> false
-		      end
-		end
-	end,
-    ejabberd_odbc:sql_transaction(LServer, F).
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    Mod:set_nick(LServer, Host, From, Nick).
 
 iq_set_register_info(ServerHost, Host, From, Nick,
 		     Lang) ->
@@ -1060,24 +799,28 @@ iq_set_register_info(ServerHost, Host, From, Nick,
 	  ErrText = <<"That nickname is registered by another "
 		      "person">>,
 	  {error, ?ERRT_CONFLICT(Lang, ErrText)};
-      _ -> {error, ?ERR_INTERNAL_SERVER_ERROR}
+      _ ->
+	  Txt = <<"Database failure">>,
+	  {error, ?ERRT_INTERNAL_SERVER_ERROR(Lang, Txt)}
     end.
 
 process_iq_register_set(ServerHost, Host, From, SubEl,
 			Lang) ->
     #xmlel{children = Els} = SubEl,
-    case xml:get_subtag(SubEl, <<"remove">>) of
+    case fxml:get_subtag(SubEl, <<"remove">>) of
       false ->
-	  case xml:remove_cdata(Els) of
+	  case fxml:remove_cdata(Els) of
 	    [#xmlel{name = <<"x">>} = XEl] ->
-		case {xml:get_tag_attr_s(<<"xmlns">>, XEl),
-		      xml:get_tag_attr_s(<<"type">>, XEl)}
+		case {fxml:get_tag_attr_s(<<"xmlns">>, XEl),
+		      fxml:get_tag_attr_s(<<"type">>, XEl)}
 		    of
 		  {?NS_XDATA, <<"cancel">>} -> {result, []};
 		  {?NS_XDATA, <<"submit">>} ->
 		      XData = jlib:parse_xdata_submit(XEl),
 		      case XData of
-			invalid -> {error, ?ERR_BAD_REQUEST};
+			invalid ->
+			      Txt = <<"Incorrect data form">>,
+			      {error, ?ERRT_BAD_REQUEST(Lang, Txt)};
 			_ ->
 			    case lists:keysearch(<<"nick">>, 1, XData) of
 			      {value, {_, [Nick]}} when Nick /= <<"">> ->
@@ -1109,7 +852,7 @@ iq_get_vcard(Lang) ->
 		[{xmlcdata,
 		  <<(translate:translate(Lang,
 					 <<"ejabberd MUC module">>))/binary,
-		    "\nCopyright (c) 2003-2015 ProcessOne">>}]}].
+		    "\nCopyright (c) 2003-2016 ProcessOne">>}]}].
 
 broadcast_service_message(Host, Msg) ->
     lists:foreach(
@@ -1192,128 +935,27 @@ opts_to_binary(Opts) ->
               Opt
       end, Opts).
 
-update_tables(Host) ->
-    update_muc_room_table(Host),
-    update_muc_registered_table(Host).
+export(LServer) ->
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    Mod:export(LServer).
 
-muc_room_schema() ->
-    {record_info(fields, muc_room), #muc_room{}}.
+import(LServer) ->
+    Mod = gen_mod:db_mod(LServer, ?MODULE),
+    Mod:import(LServer).
 
-muc_registered_schema() ->
-    {record_info(fields, muc_registered), #muc_registered{}}.
-
-update_muc_room_table(_Host) ->
-    Fields = record_info(fields, muc_room),
-    case mnesia:table_info(muc_room, attributes) of
-      Fields ->
-          ejabberd_config:convert_table_to_binary(
-            muc_room, Fields, set,
-            fun(#muc_room{name_host = {N, _}}) -> N end,
-            fun(#muc_room{name_host = {N, H},
-                          opts = Opts} = R) ->
-                    R#muc_room{name_host = {iolist_to_binary(N),
-                                            iolist_to_binary(H)},
-                               opts = opts_to_binary(Opts)}
-            end);
-      _ ->
-	  ?INFO_MSG("Recreating muc_room table", []),
-	  mnesia:transform_table(muc_room, ignore, Fields)
-    end.
-
-update_muc_registered_table(_Host) ->
-    Fields = record_info(fields, muc_registered),
-    case mnesia:table_info(muc_registered, attributes) of
-      Fields ->
-          ejabberd_config:convert_table_to_binary(
-            muc_registered, Fields, set,
-            fun(#muc_registered{us_host = {_, H}}) -> H end,
-            fun(#muc_registered{us_host = {{U, S}, H},
-                                nick = Nick} = R) ->
-                    R#muc_registered{us_host = {{iolist_to_binary(U),
-                                                 iolist_to_binary(S)},
-                                                iolist_to_binary(H)},
-                                     nick = iolist_to_binary(Nick)}
-            end);
-      _ ->
-	  ?INFO_MSG("Recreating muc_registered table", []),
-	  mnesia:transform_table(muc_registered, ignore, Fields)
-    end.
-
-export(_Server) ->
-    [{muc_room,
-      fun(Host, #muc_room{name_host = {Name, RoomHost}, opts = Opts}) ->
-              case str:suffix(Host, RoomHost) of
-                  true ->
-                      SName = ejabberd_odbc:escape(Name),
-                      SRoomHost = ejabberd_odbc:escape(RoomHost),
-                      SOpts = ejabberd_odbc:encode_term(Opts),
-                      [[<<"delete from muc_room where name='">>, SName,
-                        <<"' and host='">>, SRoomHost, <<"';">>],
-                       [<<"insert into muc_room(name, host, opts) ",
-                          "values (">>,
-                        <<"'">>, SName, <<"', '">>, SRoomHost,
-                        <<"', '">>, SOpts, <<"');">>]];
-                  false ->
-                      []
-              end
-      end},
-     {muc_registered,
-      fun(Host, #muc_registered{us_host = {{U, S}, RoomHost},
-                                nick = Nick}) ->
-              case str:suffix(Host, RoomHost) of
-                  true ->
-                      SJID = ejabberd_odbc:escape(
-                               jid:to_string(
-                                 jid:make(U, S, <<"">>))),
-                      SNick = ejabberd_odbc:escape(Nick),
-                      SRoomHost = ejabberd_odbc:escape(RoomHost),
-                      [[<<"delete from muc_registered where jid='">>,
-                        SJID, <<"' and host='">>, SRoomHost, <<"';">>],
-                       [<<"insert into muc_registered(jid, host, "
-                          "nick) values ('">>,
-                        SJID, <<"', '">>, SRoomHost, <<"', '">>, SNick,
-                        <<"');">>]];
-                  false ->
-                      []
-              end
-      end}].
-
-import(_LServer) ->
-    [{<<"select name, host, opts from muc_room;">>,
-      fun([Name, RoomHost, SOpts]) ->
-              Opts = opts_to_binary(ejabberd_odbc:decode_term(SOpts)),
-              #muc_room{name_host = {Name, RoomHost}, opts = Opts}
-      end},
-     {<<"select jid, host, nick from muc_registered;">>,
-      fun([J, RoomHost, Nick]) ->
-              #jid{user = U, server = S} =
-                  jid:from_string(J),
-              #muc_registered{us_host = {{U, S}, RoomHost},
-                              nick = Nick}
-      end}].
-
-import(_LServer, mnesia, #muc_room{} = R) ->
-    mnesia:dirty_write(R);
-import(_LServer, mnesia, #muc_registered{} = R) ->
-    mnesia:dirty_write(R);
-import(_LServer, riak, #muc_room{} = R) ->
-    ejabberd_riak:put(R, muc_room_schema());
-import(_LServer, riak,
-       #muc_registered{us_host = {_, Host}, nick = Nick} = R) ->
-    ejabberd_riak:put(R, muc_registered_schema(),
-		      [{'2i', [{<<"nick_host">>, {Nick, Host}}]}]);
-import(_, _, _) ->
-    pass.
+import(LServer, DBType, Data) ->
+    Mod = gen_mod:db_mod(DBType, ?MODULE),
+    Mod:import(LServer, Data).
 
 mod_opt_type(access) ->
-    fun (A) when is_atom(A) -> A end;
+    fun acl:access_rules_validator/1;
 mod_opt_type(access_admin) ->
-    fun (A) when is_atom(A) -> A end;
+    fun acl:access_rules_validator/1;
 mod_opt_type(access_create) ->
-    fun (A) when is_atom(A) -> A end;
+    fun acl:access_rules_validator/1;
 mod_opt_type(access_persistent) ->
-    fun (A) when is_atom(A) -> A end;
-mod_opt_type(db_type) -> fun gen_mod:v_db/1;
+    fun acl:access_rules_validator/1;
+mod_opt_type(db_type) -> fun(T) -> ejabberd_config:v_db(?MODULE, T) end;
 mod_opt_type(default_room_options) ->
     fun (L) when is_list(L) -> L end;
 mod_opt_type(history_size) ->
@@ -1327,6 +969,8 @@ mod_opt_type(max_room_id) ->
     fun (infinity) -> infinity;
 	(I) when is_integer(I), I > 0 -> I
     end;
+mod_opt_type(regexp_room_id) ->
+    fun iolist_to_binary/1;
 mod_opt_type(max_room_name) ->
     fun (infinity) -> infinity;
 	(I) when is_integer(I), I > 0 -> I
@@ -1352,7 +996,7 @@ mod_opt_type(user_presence_shaper) ->
 mod_opt_type(_) ->
     [access, access_admin, access_create, access_persistent,
      db_type, default_room_options, history_size, host,
-     max_room_desc, max_room_id, max_room_name,
+     max_room_desc, max_room_id, max_room_name, regexp_room_id,
      max_user_conferences, max_users,
      max_users_admin_threshold, max_users_presence,
      min_message_interval, min_presence_interval,
