@@ -11,7 +11,7 @@
 
 -behaviour(gen_mod).
 
--export([start/2, stop/1, muc_online_rooms/1,
+-export([start/2, stop/1, depends/2, muc_online_rooms/1,
 	 muc_unregister_nick/1, create_room/3, destroy_room/2,
 	 create_rooms_file/1, destroy_rooms_file/1,
 	 rooms_unused_list/2, rooms_unused_destroy/2,
@@ -20,42 +20,43 @@
 	 change_room_option/4, get_room_options/2,
 	 set_room_affiliation/4, get_room_affiliations/2,
 	 web_menu_main/2, web_page_main/2, web_menu_host/3,
-	 web_page_host/3, mod_opt_type/1]).
+	 web_page_host/3, mod_opt_type/1, get_commands_spec/0]).
 
 -include("ejabberd.hrl").
 -include("logger.hrl").
 -include("jlib.hrl").
 -include("mod_muc_room.hrl").
+-include("mod_muc.hrl").
 -include("ejabberd_http.hrl").
 -include("ejabberd_web_admin.hrl").
 -include("ejabberd_commands.hrl").
 
-%% Copied from mod_muc/mod_muc.erl
--record(muc_online_room, {name_host, pid}).
-
 %%----------------------------
 %% gen_mod
 %%----------------------------
 
 start(Host, _Opts) ->
-    ejabberd_commands:register_commands(commands()),
+    ejabberd_commands:register_commands(get_commands_spec()),
     ejabberd_hooks:add(webadmin_menu_main, ?MODULE, web_menu_main, 50),
     ejabberd_hooks:add(webadmin_menu_host, Host, ?MODULE, web_menu_host, 50),
     ejabberd_hooks:add(webadmin_page_main, ?MODULE, web_page_main, 50),
     ejabberd_hooks:add(webadmin_page_host, Host, ?MODULE, web_page_host, 50).
 
 stop(Host) ->
-    ejabberd_commands:unregister_commands(commands()),
+    ejabberd_commands:unregister_commands(get_commands_spec()),
     ejabberd_hooks:delete(webadmin_menu_main, ?MODULE, web_menu_main, 50),
     ejabberd_hooks:delete(webadmin_menu_host, Host, ?MODULE, web_menu_host, 50),
     ejabberd_hooks:delete(webadmin_page_main, ?MODULE, web_page_main, 50),
     ejabberd_hooks:delete(webadmin_page_host, Host, ?MODULE, web_page_host, 50).
 
+depends(_Host, _Opts) ->
+    [{mod_muc, hard}].
+
 %%%
 %%% Register commands
 %%%
 
-commands() ->
+get_commands_spec() ->
     [
      #ejabberd_commands{name = muc_online_rooms, tags = [muc],
 		       desc = "List existing rooms ('global' to get all vhosts)",
@@ -180,7 +181,8 @@ muc_online_rooms(ServerHost) ->
     MUCHost = find_host(ServerHost),
     Rooms = ets:tab2list(muc_online_room),
     lists:foldl(
-      fun({_, {Roomname, Host}, _}, Results) ->
+      fun(Room, Results) ->
+	      {Roomname, Host} = Room#muc_online_room.name_host,
 	      case MUCHost of
 		  global ->
 		      [<<Roomname/binary, "@", Host/binary>> | Results];
@@ -281,7 +283,7 @@ get_sort_query(Q) ->
 
 get_sort_query2(Q) ->
     {value, {_, String}} = lists:keysearch(<<"sort">>, 1, Q),
-    Integer = list_to_integer(binary_to_list(String)),
+    Integer = jlib:binary_to_integer(String),
     case Integer >= 0 of
 	true -> {ok, {normal, Integer}};
 	false -> {ok, {reverse, abs(Integer)}}
@@ -357,7 +359,7 @@ build_info_room({Name, Host, Pid}) ->
 	    false ->
 		Last_message1 = queue:last(History),
 		{_, _, _, Ts_last, _} = Last_message1,
-		jlib:timestamp_to_iso(Ts_last)
+		jlib:timestamp_to_legacy(Ts_last)
 	end,
 
     {<<Name/binary, "@", Host/binary>>,
@@ -397,7 +399,9 @@ prepare_room_info(Room_info) ->
 %% @spec (Name::binary(), Host::binary(), ServerHost::binary()) ->
 %%       ok | error
 %% @doc Create a room immediately with the default options.
-create_room(Name, Host, ServerHost) ->
+create_room(Name1, Host1, ServerHost) ->
+    Name = jid:nodeprep(Name1),
+    Host = jid:nodeprep(Host1),
 
     %% Get the default room options from the muc configuration
     DefRoomOpts = gen_mod:get_module_opt(ServerHost, mod_muc,
@@ -473,7 +477,7 @@ destroy_room({N, H, SH}) ->
 %% The file encoding must be UTF-8
 
 destroy_rooms_file(Filename) ->
-    {ok, F} = file:open(Filename, [read]),
+    {ok, F} = file:open(Filename, [read, binary]),
     RJID = read_room(F),
     Rooms = read_rooms(F, RJID, []),
     file:close(F),
@@ -501,23 +505,16 @@ read_room(F) ->
 %% This function is quite rudimentary
 %% and may not be accurate
 split_roomjid(RoomJID) ->
-    [Name, Host] = string:tokens(RoomJID, "@"),
-    [_MUC_service_name | ServerHostList] = string:tokens(Host, "."),
-    ServerHost = join(ServerHostList, "."),
-    {list_to_binary(Name), list_to_binary(Host), list_to_binary(ServerHost)}.
-
-%% This function is copied from string:join/2 in Erlang/OTP R12B-1
-%% Note that string:join/2 is not implemented in Erlang/OTP R11B
-join([H|T], Sep) ->
-    H ++ lists:concat([Sep ++ X || X <- T]).
-
+    [Name, Host] = binary:split(RoomJID, <<"@">>),
+    [_MUC_service_name, ServerHost] = binary:split(Host, <<".">>),
+    {Name, Host, ServerHost}.
 
 %%----------------------------
 %% Create Rooms in File
 %%----------------------------
 
 create_rooms_file(Filename) ->
-    {ok, F} = file:open(Filename, [read]),
+    {ok, F} = file:open(Filename, [read, binary]),
     RJID = read_room(F),
     Rooms = read_rooms(F, RJID, []),
     file:close(F),
@@ -694,29 +691,32 @@ send_direct_invitation(RoomName, RoomService, Password, Reason, UsersString) ->
     RoomJid = jid:make(RoomName, RoomService, <<"">>),
     RoomString = jid:to_string(RoomJid),
     XmlEl = build_invitation(Password, Reason, RoomString),
-    UsersStrings = get_users_to_invite(RoomJid, binary_to_list(UsersString)),
-    [send_direct_invitation(RoomJid, jid:from_string(list_to_binary(UserStrings)), XmlEl)
+    UsersStrings = get_users_to_invite(RoomJid, UsersString),
+    [send_direct_invitation(RoomJid, UserStrings, XmlEl)
      || UserStrings <- UsersStrings],
     timer:sleep(1000),
     ok.
 
 get_users_to_invite(RoomJid, UsersString) ->
-    UsersStrings = string:tokens(UsersString, ":"),
+    UsersStrings = binary:split(UsersString, <<":">>, [global]),
     OccupantsTuples = get_room_occupants(RoomJid#jid.luser,
 					 RoomJid#jid.lserver),
     OccupantsJids = [jid:from_string(JidString)
 		     || {JidString, _Nick, _} <- OccupantsTuples],
-    lists:filter(
-	fun(UserString) ->
-	    UserJid = jid:from_string(list_to_binary(UserString)),
-	    %% [{"badlop@localhost/work","badlop","moderator"}]
-	    lists:all(fun(OccupantJid) ->
-		UserJid#jid.luser /= OccupantJid#jid.luser
-		orelse UserJid#jid.lserver /= OccupantJid#jid.lserver
-	    end,
-	    OccupantsJids)
-	end,
-	UsersStrings).
+    lists:filtermap(
+      fun(UserString) ->
+	      UserJid = jid:from_string(UserString),
+	      Val = lists:all(fun(OccupantJid) ->
+				      UserJid#jid.luser /= OccupantJid#jid.luser
+					  orelse UserJid#jid.lserver /= OccupantJid#jid.lserver
+			      end,
+			      OccupantsJids),
+	      case Val of
+		  true -> {true, UserJid};
+		  _ -> false
+	      end
+      end,
+      UsersStrings).
 
 build_invitation(Password, Reason, RoomString) ->
     PasswordAttrList = case Password of
@@ -827,8 +827,12 @@ get_room_options(Pid) ->
     get_options(Config).
 
 get_options(Config) ->
-    Fields = record_info(fields, config),
-    [config | Values] = tuple_to_list(Config),
+    Fields = [jlib:atom_to_binary(Field) || Field <- record_info(fields, config)],
+    [config | ValuesRaw] = tuple_to_list(Config),
+    Values = lists:map(fun(V) when is_atom(V) -> jlib:atom_to_binary(V);
+                          (V) when is_integer(V) -> jlib:integer_to_binary(V);
+                          (V) when is_tuple(V); is_list(V) -> list_to_binary(hd(io_lib:format("~w", [V])));
+                          (V) -> V end, ValuesRaw),
     lists:zip(Fields, Values).
 
 %%----------------------------
@@ -884,12 +888,19 @@ make_opts(StateData) ->
     Config = StateData#state.config,
     [
      {title, Config#config.title},
+     {vcard, Config#config.vcard},
+     {voice_request_min_interval, Config#config.voice_request_min_interval},
      {allow_change_subj, Config#config.allow_change_subj},
      {allow_query_users, Config#config.allow_query_users},
      {allow_private_messages, Config#config.allow_private_messages},
+     {allow_private_messages_from_visitors, Config#config.allow_private_messages_from_visitors},
+     {allow_visitor_status, Config#config.allow_visitor_status},
+     {allow_visitor_nickchange, Config#config.allow_visitor_nickchange},
+     {allow_voice_requests, Config#config.allow_voice_requests},
      {public, Config#config.public},
      {public_list, Config#config.public_list},
      {persistent, Config#config.persistent},
+     {mam, Config#config.mam},
      {moderated, Config#config.moderated},
      {members_by_default, Config#config.members_by_default},
      {members_only, Config#config.members_only},
@@ -897,6 +908,8 @@ make_opts(StateData) ->
      {password_protected, Config#config.password_protected},
      {password, Config#config.password},
      {anonymous, Config#config.anonymous},
+     {captcha_protected, Config#config.captcha_protected},
+     {description, Config#config.description},
      {logging, Config#config.logging},
      {max_users, Config#config.max_users},
      {affiliations, ?DICT:to_list(StateData#state.affiliations)},
