@@ -6,7 +6,7 @@
 %%% Created : 18 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -33,7 +33,7 @@
 -behaviour(gen_mod).
 
 -export([start/2, stop/1, process_local_iq/3,
-	 mod_opt_type/1]).
+	 mod_opt_type/1, depends/2]).
 
 -include("ejabberd.hrl").
 -include("logger.hrl").
@@ -51,10 +51,11 @@ stop(Host) ->
 				     ?NS_TIME).
 
 process_local_iq(_From, _To,
-		 #iq{type = Type, sub_el = SubEl} = IQ) ->
+		 #iq{type = Type, sub_el = SubEl, lang = Lang} = IQ) ->
     case Type of
       set ->
-	  IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]};
+	  Txt = <<"Value 'set' of 'type' attribute is not allowed">>,
+	  IQ#iq{type = error, sub_el = [SubEl, ?ERRT_NOT_ALLOWED(Lang, Txt)]};
       get ->
 	  Now_universal = calendar:universal_time(),
 	  Now_local = calendar:universal_time_to_local_time(Now_universal),
@@ -85,5 +86,8 @@ process_local_iq(_From, _To,
 sign(N) when N < 0 -> <<"-">>;
 sign(_) -> <<"+">>.
 
+depends(_Host, _Opts) ->
+    [].
+
 mod_opt_type(iqdisc) -> fun gen_iq_handler:check_type/1;
 mod_opt_type(_) -> [iqdisc].
