@@ -6,17 +6,17 @@ def read():
     pkt = sys.stdin.read(pkt_size).split(':')
     cmd = pkt[0]
     args_num = len(pkt) - 1
-    if cmd == 'auth' and args_num == 3:
+    if cmd == 'auth' and args_num >= 3:
         write(True)
     elif cmd == 'isuser' and args_num == 2:
         write(True)
-    elif cmd == 'setpass' and args_num == 3:
+    elif cmd == 'setpass' and args_num >= 3:
         write(True)
-    elif cmd == 'tryregister' and args_num == 3:
+    elif cmd == 'tryregister' and args_num >= 3:
         write(True)
     elif cmd == 'removeuser' and args_num == 2:
         write(True)
-    elif cmd == 'removeuser3' and args_num == 3:
+    elif cmd == 'removeuser3' and args_num >= 3:
         write(True)
     else:
         write(False)
